(function () {
'use strict'



/*Filters*/
Vue.filter('bytes', function (bytes) {
	if (typeof bytes === 'string') {
		return bytes
	}

	if (!(bytes > 0)) {
		return '-'
	}

	var units = ['B', 'KB', 'MB', 'GB', 'TB'],
		level = parseInt(Math.floor(Math.log(bytes) / Math.log(1024))),
		count = bytes / Math.pow(1024, level)

	if (count < 10) {
		count = count.fix(2)
	}
	else if (count < 100) {
		count = count.fix(1)
	}
	else {
		count = count.fix(0)
	}

	return count + ' ' + units[level]
})



Vue.filter('duration', function (time) {
	var hour = Math.floor(time / 3600),
		min = Math.floor(time % 3600 / 60) || 0,
		sec = Math.floor(time % 60) || 0

	return (hour ? hour.fillTo(2) + ':' : '') + min.fillTo(2) + ':' + sec.fillTo(2)
})



Vue.filter('date', function (date) {
	if (date < 1e13) {
		var datePow = Math.ceil(Math.log(date) / Math.LN10)
		date *= Math.pow(10, 13 - datePow)
	}
	
	return new Date(date).format('y-M-d h:m')
})



Vue.filter('shortdate', function (date) {
	if (!date) {
		return '-'
	}

	if (date < 1e13) {
		var datePow = Math.ceil(Math.log(date) / Math.LN10)
		date *= Math.pow(10, 13 - datePow)
	}

	var d = new Date(date),
		n = new Date()

	if (d.getFullYear() === n.getFullYear() && d.getMonth() === n.getMonth() && d.getDate() === n.getDate()) {
		return d.format('h:m')
	}
	else if (d.getFullYear() === n.getFullYear()) {
		return d.format('M-d h:m')
	}
	else {
		return d.format('y-M-d h:m')
	}
})



Vue.filter('ext', function (path) {
	return String(path).select(/\.([a-z0-9]+)$/i, 1)
})



/*Transitions*/
Vue.transition('collapse', {

	enter: function (el, done) {
		el.show()
		var finalHeight = el.offsetHeight
		if (!finalHeight) {
			return done()
		}

		el.css('height', 0)

		var ontransitionend = function () {
			el.css('height', 'auto')
		}

		el.redraw()
		el.css('height', finalHeight)
		el.once('transitionend', ontransitionend)

		return function () {
			ontransitionend()
			el.off('transitionend', ontransitionend)
		}
	},


	leave: function (el, done) {
		var startHeight = el.offsetHeight
		if (!startHeight) {
			return done()
		}

		el.css('height', startHeight)

		var ontransitionend = function () {
			el.css('height', 'auto')
			el.hide()
		}

		el.redraw()
		el.css('height', 0)
		el.once('transitionend', ontransitionend)

		return function () {
			ontransitionend()
			el.off('transitionend', ontransitionend)
		}
	}
})



/*Directives*/
Vue.directive('loading', {

	template: ['<div class="loading-mask">',
		'<div class="loading-anim"></div>',
	'</div>'].join('\n'),

	maskEl: null,

	update: function (value) {
		if (value) {
			this.el.on('boxchange', this.alignMask, this, true)
		}
		else {
			if (this.maskEl) {
				this.maskEl.destroy()
				this.maskEl = null
			}
			this.el.off('boxchange', this.alignMask, this)
		}
	},


	alignMask: function () {
		var box = this.el.box(true)
		
		if (box.width && box.height) {
			if (this.maskEl) {
				this.maskEl.show()
			}
			else {
				this.maskEl = document.create(this.template)
				document.body.append(this.maskEl)
			}
			
			this.maskEl.css(box)
		}
		else if (this.maskEl) {
			this.maskEl.hide()
		}
	},


	unbind: function () {
		this.update(false)
	}
})



Vue.directive('tip', {

	tipValue: '',

	tipEl: null,

	margin: 14,

	bind: function () {
		this.el.on('mouseenter', this.showTip, this)
		this.el.on('mouseleave', this.hideTip, this)

		var hasMargin = this.el.hasAttribute('margin')
		if (hasMargin) {
			this.margin = this.el.getAttribute('margin')
			this.el.removeAttribute('margin')
		}
	},


	unbind: function () {
		this.el.off('mouseenter', this.showTip, this)
		this.el.off('mouseleave', this.hideTip, this)
	},


	update: function (value) {
		this.tipValue = value

		if (this.tipEl) {
			this.tipEl.query('.tip-value').innerHTML = value
			this.alignTip()
		}
	},


	showTip: function () {
		if (!this.tipValue) {
			return
		}

		var tip = this.tipEl
		if (this.tipEl) {
			this.tipEl.anim('fade', true)
		}
		else{
			this.tipEl = document.create(
				['<div class="tip">',
					'<div class="triangle"></div>',
					'<div class="tip-value">{0}</div>',
				'</div>']
				.join('')
				.format(this.tipValue)
			)

			this.tipEl.anim('fade', true)
			document.body.append(this.tipEl)
			this.alignTip()
		}
	},


	alignTip: function () {
		var margin = this.margin

		this.tipEl.alignTo(this.el, 'bc-tc', margin)
	},


	hideTip: function () {
		var tip = this.tipEl
		if (tip) {
			tip.anim('fade', false, function () {
				tip.destroy()
				this.tipEl = null
			}, this)
		}
	}
})



Vue.directive('lazysrc', {

	loadedClass: 'loaded',

	defaultSrc: '',

	destroyPreload: null,

	bind: function () {
		var el = this.el
		if (el.hasAttribute('defaultsrc')) {
			this.defaultSrc = el.getAttribute('defaultsrc')
			el.removeAttribute('defaultsrc')
		}
	},


	unbind: function () {
		if (this.destroyPreload) {
			this.destroyPreload()
			this.destroyPreload = null
		}
	},


	update: function (value) {
		this.el.removeClass(this.loadedClass)
		this.destroyPreload = ff.preload(value, this.onLoad, this)
	},


	onLoad: function (img) {
		this.destroyPreload = null
		
		if (img) {
			if (this.el.tagName === 'IMG') {
				this.el.src = img.src
			}
			else {
				this.el.style.backgroundImage = 'url(' + img.src + ')'
			}

			this.el.addClass(this.loadedClass)
		}
		else {
			if (this.el.tagName === 'IMG') {
				this.el.src = this.defaultSrc
			}
			else {
				this.el.style.backgroundImage = this.defaultSrc ? 'url(' + this.defaultSrc + ')' : 'none'
			}

			this.el.removeClass(this.loadedClass)
		}
	}
})



//object-fit: contain
Vue.directive('fit', {

	isLiteral: true,

	width: 0,

	height: 0,

	bind: function () {
		var el = this.el
		this.width = el.offsetWidth
		this.height = el.offsetHeight

		if (!this.width || !this.height) {
			console.warn('`{0}` element\'s width and height cant be 0!')
		}

		this.onWindowResize()
		window.on('resize', this.onWindowResize, this)
	},


	unbind: function () {
		window.off('resize', this.onWindowResize, this)
	},


	onWindowResize: function () {
		var el = this.el
		var w = el.parentNode.clientWidth

		el.css('width', w)
		el.css('height', w * this.height / this.width)
	}
})



Vue.directive('copy', {

	copyText: '',

	bind: function () {
		var el = this.el
		el.on('click', this.onClickEl, this)
	},

	update: function (value) {
		this.copyText = value
	},

	onClickEl: function () {
		if (this.copyText && document.execCommand) {
			var input = document.create('<input style="position: fixed; left: 0; top: 0; z-index: -1;">')
			document.body.append(input)
			input.value = this.copyText
			input.select()

			if (document.execCommand('copy')) {
				ff.notify.success('copied_to_clipboard')
			}
			else {
				ff.notify.error('copy_clipboard_failed')
			}

			input.remove()
		}
	},

	unbind: function () {
		window.off('resize', this.onWindowResize, this)
	},
})



//el:
//drop
//	label
//	ul
Vue.component('drop', {

	replace: true,

	props: ['align', 'margin', 'menu-class'],

	data: {
		active: false,

		align: 'l',

		margin: 8,

		menuClass: '',
	},


	template: ['<div class="drop" f-class="active: active" f-on="click: onClick">',
		'<div class="drop-label"><content select="label" replace></content></div>',
		'<div class="menu drop-menu" f-el="menu" f-show="active" hidden f-transition="fade">',
			'<div class="triangle" f-el="triangle"></div>',
			'<content select="ul"></content>',
		'</div>',
	'</div>'].join('\n'),


	ready: function () {
		document.body.append(this.$$.menu)
		
		this.$$.menu.on('click', function () {
			this.active = false
		}, this)

		if (this.menuClass) {
			this.$$.menu.addClass(this.menuClass)
		}
	},


	methods: {
		onClick: function (e) {
			e.stopPropagation()

			if (!this.active) {
				this.active = true
				Vue.flush()
				this.onWindowResize()
				this.$el.on('boxchange', this.onWindowResize, this)
				document.on('mousedown', this.onDocMouseDown, this)
			}
			else {
				this.active = false
			}
		},


		onWindowResize: function () {
			var pos = 't{0}-b{0}'.format(this.align)
			
			this.$$.menu.alignTo(this.$el, pos, this.margin, 'y')
		},


		onDocMouseDown: function (e) {
			if (!e.within(this.$el) && !e.within(this.$$.menu)) {
				this.active = false
			}
		}
	},


	watch: {
		active: function (active) {
			if (!active) {
				//window.off('resize', this.onWindowResize, this)
				this.$el.off('boxchange', this.onWindowResize, this)
				document.off('mousedown', this.onDocMouseDown, this)
			}
		}
	}
})



//trigger when hover
//not support enter and leave event on iphone safari
//not trigger mouseout on ipad safari
//we need a timer to make sure when mouse move from btn to layer, layer not hidden
Vue.component('hdrop', Vue.component('drop').extend({

	template: ['<div class="drop" f-class="active: active" f-on="mouseover: onMouseOver, mouseout: onMouseOut">',
		'<div class="drop-label"><content select="label" replace></content></div>',
		'<div class="menu drop-menu" f-el="menu" f-show="active" hidden f-transition="fade"',
			' f-on="mouseover: onMenuOver, mouseout: onMenuOut">',
			'<div class="triangle" f-el="triangle"></div>',
			'<content select="ul"></content>',
		'</div>',
	'</div>'].join('\n'),


	methods: {
		clearTimer: function () {
			if (this.timer) {
				this.timer.close()
				this.timer = null
			}
		},

		onMouseOver: function (e) {
			this.clearTimer()
			this.active = true
			Vue.flush()
			this.onWindowResize()
			document.on('mousedown', this.onDocMouseDown, this)
		},


		onMouseOut: function (e) {
			this.clearTimer()
			this.timer = new ff.Timer(100, true, function () {
				this.active = false
				this.timer = null
			}, this)
		},


		onMenuOver: function (e) {
			this.clearTimer()
			this.active = true
		},


		onMenuOut: function (e) {
			this.onMouseOut(e)
		}
	}
}))



//events:
//	change: trigger when value change
//	select: trigger when selected from menu
Vue.component('select', Vue.component('drop').extend({

	replace: true,

	data: {
		data: null,

		value: '',

		default: undefined,	//if can't found, use this value

		displayValue: '',

		key: '',	//support $key, $value, default is empty string, use key

		display: '',

		suggest: '',

		isInputing: false,

		readonly: false,

		placeholder: '',

		keyDisplayData: null,

		filteredKeyDisplayData: null,

		margin: 0,

		order: 0,
	},

	props: ['data', 'value', 'key', 'display', 'readonly', 'placeholder', 'default', 'order'],

	template: ['<div class="select" f-class="active: active">',
		'<input class="select-input" type="text"',
			'f-attr="readonly: readonly, placeholder: placeholder"',
			'f-model="suggest"',
			'f-on="click: onClick, focus: onFocus, blur: onBlur, input: onInput">',
		'<div class="menu select-menu" f-el="menu" f-show="active" hidden f-cloak>',
			'<ul>',
				'<li f-repeat="isInputing ? filteredKeyDisplayData : keyDisplayData"',
					'f-on="click: onMenuClick($value)"',
					'f-class="active: keyValue == value">',
					'{{displayValue}}',
				'</li>',
			'</ul>',
		'</div>',
	'</div>'].join('\n'),


	ready: function () {
		document.body.append(this.$$.menu)
	},


	methods: {
		buildKeyDisplayData: function () {
			var kdData = [],
				data = this.data,
				isInputing = this.isInputing,
				suggest = this.suggest.toLowerCase(),
				key = this.key,
				display = this.display

			ff.each(data, function(item, i) {
				var keyValue = key === '$key' ? i
					: key === '$value' ? String(item)
					: key ? String(ff.get(item, key))
					: String(i)

				var displayValue = display === '$key' ? i
					: display === '$value' ? String(item)
					: display ? String(ff.get(item, display))
					: String(item)

				kdData.push({
					keyValue: keyValue,
					displayValue: displayValue
				})
			})

			if (this.order == 1) {
				kdData = kdData.sortBy('displayValue')
			}
			else if (this.order == -1) {
				kdData = kdData.sortBy('displayValue', true)
			}

			this.keyDisplayData = kdData
		},


		onMenuClick: function (item) {
			this.value = item.keyValue
			this.isInputing = false
			this.active = false

			this.$trigger('select', this.value)
		},


		onFocus: function () {
			this.$$.menu.css('min-width', this.$el.offsetWidth)
		},


		onBlur: function () {
			this.suggest = this.displayValue
		},


		onInput: function (e) {
			//bug on IE, input trigger input event when not focused
			if (document.activeElement && document.activeElement !== e.target) {
				return
			}

			this.isInputing = true
			this.active = true

			var suggest = this.suggest.toLowerCase()
			this.filteredKeyDisplayData = this.keyDisplayData.filter(function (pair) {
				return pair.displayValue.toLowerCase().contains(suggest)
			})
		},


		findDisplayValue: function () {
			var value = this.value,
				defaultValue = this.default,
				keyDisplayData = this.keyDisplayData

			if (keyDisplayData) {
				var matchPair = keyDisplayData.select('keyValue', value)
					|| keyDisplayData.select('keyValue', defaultValue)

				if (matchPair){
					this.suggest = this.displayValue = matchPair.displayValue
				}
			}
		}
	},


	watch: {
		data: function () {
			this.buildKeyDisplayData()
			this.findDisplayValue()
			Vue.flush()
			this.onWindowResize()
		},


		value: function (value) {
			this.findDisplayValue()
			this.$trigger('change', this.value)
		}
	}
}))



//events:
//	change: trigger when selected from menu
Vue.component('ckselect', Vue.component('select').extend({

	replace: true,

	data: {
		value: [],

		displayValue: [],

		readonly: true,

		//not support default
	},

	template: ['<div class="select" f-class="active: active">',
		'<input class="select-input" type="text"',
			'f-attr="readonly: readonly, placeholder: placeholder"',
			'f-model="suggest"',
			'f-on="click: onClick, focus: onFocus, input: onInput">',
		'<div class="menu select-menu" f-el="menu" f-show="active" hidden f-cloak>',
			'<ul>',
				'<li f-repeat="keyDisplayData"',
					'f-on="click: onMenuClick($value)"',
					'f-class="active: contains(keyValue)">',
					'<div class="checkbox" f-class="checked: contains(keyValue)">{{displayValue}}</div>',
				'</li>',
			'</ul>',
		'</div>',
	'</div>'].join('\n'),


	ready: function () {
		if (!this.value) {
			this.value = []
		}
	},


	methods: {

		onMenuClick: function (item) {
			var value = item.keyValue,
				values = this.value

			if (values.contains(value)) {
				values.remove(value)
			}
			else {
				values.push(value)
			}

			this.$trigger('select', values)
		},


		contains: function (value) {
			var values = this.value

			for (var i = 0; i < values.length; i++) {
				if (values[i] == value) {
					return true
				}
			}
		},


		onBlur: function () {
			this.suggest = this.displayValue.join('; ')
		},


		findDisplayValue: function () {
			var values = this.value,
				displayValues = [],
				keyDisplayData = this.keyDisplayData

			if (keyDisplayData) {
				for (var i = 0; i < values.length; i++) {
					var value = values[i],
						matchPair = keyDisplayData.select('keyValue', value)

					if (matchPair){
						displayValues.push(matchPair.displayValue)
					}
				}
			}

			this.displayValue = displayValues
			this.suggest = displayValues.join('; ')
		}
	}
}))



Vue.component('cmselect', Vue.component('select').extend({

	replace: true,

	data: {
		custom: '',

		customValue: '',
	},

	props: ['custom', 'custom-value'],

	methods: {
		onMenuClick: function (item) {
			this.value = item.keyValue
			this.isInputing = false
			this.active = false

			this.$trigger('select', this.value)

			if (this.value == this.custom) {
				ff.msg.prompt('input_custom_name', function (btn, value) {
					if (btn === 'ok') {
						value = value.trim()
						this.customValue = value
					}
				}, this)
			}
		},


		onBlur: function () {
			this.suggest = this.value == this.custom ? this.customValue : this.displayValue
		},


		findDisplayValue: function () {
			var value = this.value,
				defaultValue = this.default,
				keyDisplayData = this.keyDisplayData

			if (keyDisplayData) {
				var matchPair = keyDisplayData.select('keyValue', value)
					|| keyDisplayData.select('keyValue', defaultValue)

				if (matchPair){
					this.suggest = this.displayValue = this.value == this.custom ? this.customValue : matchPair.displayValue
				}
			}
		}
	},


	watch: {
		custom: function (custom) {
			if (this.value == this.custom) {
				this.suggest = this.displayValue = this.customValue
			}
		},


		customValue: function (customValue) {
			if (this.value == this.custom) {
				this.suggest = this.displayValue = this.customValue
			}
		}
	}
}))



//define a language select component
//once initialized, register a filter trans
//can only use local trans, can't use remote trans now
//events:
//	change(lang)
Vue.component('lang', Vue.component('drop').extend({

	data:  {
		defaultLanguage: document.documentElement.lang || 'en',

		currentLanguage: '',

		languageString: 'en cn',

		gotLangFromStorage: false,

		languages: null,

		dir: '',

		align: 'r',

		margin: 0,
	},


	props: ['dir', 'language-string'],


	template: ['<div class="drop lang-select" f-el="select" f-class="active: active" f-on="click: onClick">',
		'<div class="drop-label" f-class="currentLanguage"></div>',
		'<div class="menu lang-menu" f-el="menu" f-show="active" hidden f-transition="fade">',
			'<div class="triangle" f-el="triangle"></div>',
			'<ul>',
				'<li f-repeat="languages" class="{{$value}}"',
					'f-class="active: currentLanguage === $value"',
					'f-on="click: onClickMenu($value, $event)"></li>',
			'</ul>',
		'</div>',
	'</div>'].join('\n'),

	compiled: function () {
		if (this.languageString) {
			this.languages = this.languageString.split(' ')
		}
	},


	ready: function () {
		var options = this.$options

		this.cache = {}	//default: {}	//if we want to use a default trans from html codes

		var els = document.queryAll('[trans]')

		for (var i = 0; i < els.length; i++) {
			var el = els[i],
				key = el.attr('trans'),
				text = el.textContent.trim()

			if (key) {
				key = key.replace(/\s+/g, '_').toLowerCase()
			}
			else {
				key = text.replace(/\s+/g, '_').toLowerCase()
				el.attr('trans', key)
			}

			// if (text) {
			// 	this.cache['default'][key] = text
			// }
		}

		var languages = this.languages
		var defaultLang = this.defaultLanguage
		var navLangs = (navigator.language || '').toLowerCase().split('-')
		var currLang = ff.store.get('lang')

		this.cache[defaultLang] = require('lang.' + defaultLang)

		if (!currLang || !languages.contains(currLang)) {
			if (navLangs[0] && languages.contains(navLangs[0])) {
				currLang = navLangs[0]
			}
			else if (navLangs[1] && languages.contains(navLangs[1])) {
				currLang = navLangs[1]
			}
			else {
				currLang = defaultLang
			}
		}
		else {
			this.gotLangFromStorage = true
		}
		
		this.changeLanguageTo(currLang)
	},


	methods: {
		get: function (key) {
			if (!key) {
				return ''
			}

			var value

			if (key.contains('@@')) {
				value = key.slice(0, key.indexOf('@@'))
				key = key.slice(key.indexOf('@@') + 2)
			}

			var lowerKey = key.replace(/[\s-]+/g, '_').toLowerCase()

			var hash = this.cache[this.currentLanguage]
			if (hash && lowerKey in hash) {
				return hash[lowerKey]
			}

			var hash = this.cache[this.defaultLanguage]
			if (hash && lowerKey in hash) {
				return hash[lowerKey]
			}

			return value || key
		},


		translate: function (key, data) {
			var trans = this.get(key)

			if (data !== undefined && data !== null) {
				if (typeof data !== 'object') {
					data = ff.toArray(arguments)
					data.shift()
				}
				trans = trans.format(data)
			}

			return trans
		},


		translateElement: function (el, data) {
			var els

			if (el.hasAttribute('trans')) {
				els = [el]
			}
			else {
				els = el.queryAll('[trans]')
			}

			for (var i = 0; i < els.length; i++) {
				var el = els[i]
				var key = el.attr('trans')

				if (key && /\{\{.+?\}\}/.test(key)) {
					console.warn('You may try to translate interpolater `{0}`!'.format(key))
				}

				el.innerHTML = this.translate(key, data)
			}
		},


		changeLanguageTo: function (lang) {
			if (require.cache['lang.' + lang] && !this.cache[lang]) {
				this.cache[lang] = require('lang.' + lang)
			}

			if (lang in this.cache) {
				this.currentLanguage = lang
				ff.store.set('lang', lang)
				document.documentElement.lang = lang
				this.translateElement(document.documentElement)
				Vue.flush()
				this.$trigger('change', lang)
			}
			else {
				console.warn('Cant find language `{0}`'.format(lang))
			}
		},


		hasLanguage: function (lang) {
			return !!this.languages.contains(lang)
		},


		onClickMenu: function (key, e) {
			e.stopPropagation()
			this.changeLanguageTo(key)
			this.active = false
		}
	}
}))



//when document is ready, parse lang element to component and register a trans filter
ff.ready(function () {
	var langEl = document.query('lang')
	if (langEl) {
		ff.lang = new (Vue.component('lang'))({
			el: langEl
		})

		Vue.filter('trans', ff.lang.translate.bind(ff.lang))
	}
	else {
		Vue.filter('trans', function (trans, data) {
			if (data !== undefined && data !== null) {
				if (typeof data !== 'object') {
					data = ff.toArray(arguments)
					data.shift()
				}
				
				trans = trans.format(data)
			}

			return trans
		})
	}
})



//events:
//	change
Vue.component('slider', {

	replace: true,

	data: {
		min: 0,

		max: 100,

		step: 1,

		value: 50, 
	},

	props: ['min', 'max', 'step', 'value'],

	template: ['<div class="slider" f-on="mousedown: onMouseDown, mousewheel: onMouseWheel" f-el="volume">',
		'<div class="slider-inner">',
			'<div class="slider-per" f-style="width: perWidth()">',
				'<div class="slider-ball"></div>',
			'</div>',
		'</div>',
	'</div>'].join('\n'),


	methods: {
		perWidth: function () {
			if (!(this.min < this.max)) {
				return '0'
			}
			else {
				var per = (this.value - this.min) / (this.max - this.min) * 100
				return per.constrain(0, 100) + '%'
			}
		},


		onMouseDown: function (e) {
			var box = this.$el.box()
			this.changeValueByEvent(e, box)

			var onmousemove = function (e) {
				this.changeValueByEvent(e, box)
			}

			document.on('mousemove', onmousemove, this)

			document.once('mouseup', function () {
				document.off('mousemove', onmousemove, this)
			}, this)
		},


		changeValueByEvent: function (e, box) {
			var x = e.pageX,
				per = (x - box.left) / box.width,
				dif

			per = per.constrain(0, 1)
			dif = (this.max - this.min) * per

			if (this.step) {
				dif = Math.round(dif / this.step) * this.step
			}

			var oldValue = this.value
			this.value = this.min + dif

			if (this.value !== oldValue) {
				this.$trigger('change', this.value)
			}
		},


		onMouseWheel: function (e) {
			e.stopPropagation()

			if (!this.step) {
				return
			}

			var oldValue = this.value

			if (e.wheelDelta > 0 || e.detail < 0) {
				this.value = Math.max(this.value - this.step, this.min)
			}
			else {
				this.value = Math.min(this.value + this.step, this.max)
			}

			if (this.value !== oldValue) {
				this.$trigger('change', this.value)
			}
		}
	}
})



Vue.component('switch', {

	replace: true,

	data: {
		value: false, 
	},

	props: ['value'],

	template: ['<div class="switch" f-class="active: value" f-on="click: onClick">',
		'<div class="switch-ball"></div>',
	'</div>'].join('\n'),

	methods: {
		onClick: function () {
			this.value = !this.value
			this.$trigger('change', this.value)
		}
	}
})



ff.winManager = {

	//should same as the z-index value specified in the .win style
	startIndex: 100,

	index: 100,

	maxIndex: 110,

	wins: [],

	lastWin: null,

	register: function (win) {
		var mag = ff.winManager

		win.$$.win.on('mousedown', mag.onWinMouseDown.bind(mag, win), true)
		win.$watch('hidden', function (hidden) {
			if (hidden === false) {
				mag.onWinMouseDown(win)
			}
		})
		win.$on('hook:destroyed', mag.onWinDestroyed, win)
		mag.onWinMouseDown(win)
	},


	onWinMouseDown: function (win) {
		var mag = ff.winManager

		if (win !== mag.lastWin) {
			mag.wins.remove(win)
			mag.wins.push(win)
			mag.lastWin = win
			win.$el.css('z-index', mag.index++)

			if (mag.index > mag.maxIndex) {
				mag.recycleIndex()
			}
		}
	},


	recycleIndex: function () {
		var mag = ff.winManager

		mag.index = mag.startIndex
		for (var i in mag.wins) {
			var win = mag.wins[i]
			win.$el.css('z-index', mag.index++)
		}
	},


	onWinDestroyed: function () {
		var win = this,
			mag = ff.winManager

		mag.wins.remove(win)
		if (mag.lastWin === win) {
			mag.lastWin = mag.wins.last() || null
		}
	}
}



//events:
//	show
//	hide
Vue.component('win', {

	replace: true,

	beManaged: true,

	data: {
		hidden: true,

		autoDestroy: false,
	},

	//extensions may make win wrapped by a mask, so we need a win el
	template: ['<div class="win" f-el="win" f-hide="hidden" f-transition="fade">',
		'<div class="win-head" f-el="head"><content select="whead" replace></content></div>',
		'<div class="win-close" f-el="close" f-on="click: hide"></div>',
		'<div class="win-body" f-el="body"><content select="wbody" replace></content></div>',
		'<div class="win-foot" f-el="foot"><content select="wfoot" replace></content></div>',
	'</div>'].join('\n'),

	ready: function () {
		this.$$.head.makeDragable(this.$$.win)
		if (this.$options.beManaged) {
			ff.winManager.register(this)
		}
		window.on('resize', this.onWindowResize, this)
		this.onWindowResize()
	},


	destroyed: function () {
		window.off('resize', this.onWindowResize, this)
	},


	methods: {
		//will not show immediately
		show: function () {
			this.hidden = false
		},


		hide: function () {
			this.hidden = true
		},


		center: function () {
			this.$$.win.alignTo(document.body, 'c-c')
		},


		toggle: function () {
			this.hidden ? this.show() : this.hide()
		},


		toFront: function () {
			ff.winManager.onWinMouseDown(this)
		},


		onWindowResize: function () {
			if (!this.hidden) {
				this.$$.body.css('width', 'auto')
				this.$$.body.css('height', 'auto')
				
				var verGap = document.documentElement.offsetHeight - this.$$.win.offsetHeight - 40
				if (verGap < 0) {
					this.$$.body.css('height', this.$$.body.offsetHeight + verGap)
				}

				var herGap = document.documentElement.offsetWidth - this.$$.win.offsetWidth - 40
				if (herGap < 0) {
					this.$$.body.css('width', this.$$.body.offsetWidth + herGap)
				}

				this.center()
			}
		}
	},

	watch: {
		hidden: function(hidden) {
			if (!hidden) {
				this.onWindowResize()
			}

			if (hidden && this.autoDestroy) {
				this.$destroy(true)
			}

			if (hidden) {
				this.$trigger('hide')
			}
			else {
				this.$trigger('show')
			}
		}
	}
})



Vue.component('msg-win', Vue.component('win').extend({

	beManaged: false,

	data: {

		title: '',

		msg: '',

		args: null,

		buttons: null,

		isPrompt: false,

		value: '',

		cls: '',
	},

	template: ['<div class="msg-mask" f-hide="hidden">',
		'<div class="win msg" f-el="win" f-hide="hidden">',
			'<div class="win-close" f-el="close" f-on="click: onClickClose"></div>',
			'<div class="win-head" f-el="head">{{title | trans}}</div>',
			'<div class="win-body" f-el="body">',
				'<div class="msg-body" f-class="cls">',
					'<div class="msg-con" f-html="getMsg()"></div>',
					'<div class="msg-input-wrap" f-show="isPrompt">',
						'<input class="msg-input" f-el="input" type="text" f-model="value" f-on="keydown: onKeyDown">',
					'</div>',
				'</div>',
			'</div>',
			'<div class="win-foot" f-el="foot">',
				'<button f-repeat="buttons"',
					'f-on="click: onClickBtn($key)"',
					'f-class="btnCls"',
					'f-disable="isPrompt && !value.trim()">',
					'{{label | trans}}',
				'</button>',
			'</div>',
		'</div>',
	'</div>'].join('\n'),

	methods: {
		onClickBtn: function (btn) {
			this.$trigger('btnclick', btn)
		},


		onClickClose: function () {
			this.hide()
			this.$trigger('btnclick', 'cancel')
		},


		onKeyDown: function (e) {
			if (e.keyCode === 10 || e.keyCode === 13) {
				this.onClickBtn('ok')
			}
		},


		getMsg: function () {
			return Vue.filter('trans')(this.msg, this.args)
		}
	}
}))


/*ff.msg.show({
	msg: start message
	args: arguments to format msg
	buttons: {'ok', 'cancel'}
	handler: function (btn), -1 is close
	type | cls: 
	scope
})

button label will be automatically translated
*/
ff.msg = {
	//cache all the cfg that not showed
	stack : [],

	dlg: null,

	seed: 1,

	getDialog : function () {
		if (!this.dlg) {
			this.dlg = new (Vue.component('msg-win'))({
				el: document.createElement('div'),
				events: {
					btnclick: this.onBtnClick.bind(this)
				}
			})
			this.dlg.$appendTo(document.body)

			if (ff.lang) {
				ff.lang.$on('change', this.onLangChange.bind(this))
			}
		}
	},


	onLangChange: function () {
		if (!this.dlg.hidden) {
			this.dlg.center()
		}
	},


	onBtnClick : function (btn) {
		if (this.dlg.isPrompt && !this.dlg.$$.input.validity.valid) {
			this.dlg.$$.input.checkValidity()
		}
		else{
			this.dlg.hide()
			
			var value = this.dlg.isPrompt ? this.dlg.value : undefined

			if (this.cfg.handler) {
				this.cfg.handler.call(this.cfg.scope, btn, value)
			}
			if (this.stack.length > 0) {
				this.show(this.stack.shift())
			}
		}
	},


	show: function (cfg) {
		cfg.id = this.seed++

		this.getDialog()
		
		if (this.dlg.hidden) {
			this.cfg = cfg
			cfg.isPrompt = Boolean(cfg.isPrompt)
			cfg.value = cfg.value || cfg.value === 0 ? cfg.value : ''

			if (cfg.type && !cfg.cls) {
				cfg.cls = 'msg-' + cfg.type
			}

			for (var i in cfg.buttons) {
				if (typeof cfg.buttons[i] === 'string') {
					cfg.buttons[i] = {
						label: cfg.buttons[i]
					}
				}

				cfg.buttons[i].btnCls = cfg.buttons[i].btnCls || ''
			}

			ff.apply(this.dlg, cfg, ['title', 'msg', 'args', 'buttons', 'isPrompt', 'value', 'cls'])
			this.dlg.show()

			Vue.flush()
			this.dlg.center()

			if (cfg.isPrompt) {
				this.dlg.$$.input.focus()
			}
		}
		else{
			this.stack.push(cfg)
		}

		return cfg.id
	},


	hide: function (id) {
		if (!id || id && this.cfg && this.cfg.id === id) {
			if (this.dlg) {
				this.dlg.hide()
			}
			
			if (this.stack.length > 0) {
				this.show(this.stack.shift())
			}
		}
		else if (id) {
			var cfg = this.stack.select('id', id)
			if (cfg) {
				this.stack.remove(cfg)
			}
		}
	},


	clear: function () {
		this.stack = []
		this.hide()
	},


	info: ff.flex('[string] string [object]', function (title, msg, args, handler, scope) {
		return this.show({
			title: title || 'Info',
			msg: msg,
			args: args,
			handler: handler,
			scope: scope,
			type: 'info',
			buttons: {ok: 'Ok'}
		})
	}),


	success: ff.flex('[string] string [object]', function (title, msg, args, handler, scope) {
		return this.show({
			title: title || 'Info',
			msg: msg,
			args: args,
			handler: handler,
			scope: scope,
			type: 'success',
			buttons: {ok: 'Ok'}
		})
	}),


	confirm: ff.flex('[string] string [object]', function (title, msg, args, handler, scope) {
		return this.show({
			title: title  || 'Confirm',
			msg: msg,
			args: args,
			handler: handler,
			scope: scope,
			type: 'confirm',
			buttons: {yes: {label: 'Yes', btnCls: 'blue'}, no: 'No'}
		})
	}),


	alert: ff.flex('[string] string [object]', function (title, msg, args, handler, scope) {
		return this.show({
			title: title || 'Alert',
			msg: msg,
			args: args,
			handler: handler,
			scope: scope,
			type: 'alert',
			buttons: {ok: 'Ok'}
		})
	}),


	prompt: ff.flex('[string] string [object] [string|number]', function (title, msg, args, value, handler, scope) {
		return this.show({
			title: title || 'Prompt',
			msg: msg,
			args: args,
			handler: handler,
			scope: scope,
			isPrompt: true,
			value: value,
			type: 'prompt',
			buttons: {ok: 'Ok'}
		})
	}),
}



Vue.component('notify', {

	replace: true,

	data: {
		seed: 1,

		stack: null,
	},

	template: ['<div class="notify">',
		'<div class="notify-item" f-repeat="stack" f-transition="fade" f-class="cls" f-on="click: onClick($value)">',
			'<div class="notify-msg" f-html="getMsg($value)"></div>',
			'<div class="notify-close"></div>',
		'</div>',
	'</div>'].join('\n'),


	ready: function () {
		document.body.append(this.$el)
	},


	methods: {
		onClick: function (item) {
			this.stack.remove(item)
		},


		getMsg: function (item) {
			return Vue.filter('trans')(item.msg, item.args)
		}
	}
})



ff.notify = {

	seed: 1,

	timeout: 3000,

	stack: [],

	getDialog: function () {
		if (!this.dlg) {
			this.dlg = new (Vue.component('notify'))({
				el: document.createElement('div'),
				data: {
					stack: this.stack
				}
			})
			this.dlg.$appendTo(document.body)
		}
	},


	show: function (cfg) {
		this.getDialog()

		cfg.timeout = cfg.timeout || this.timeout

		var reuse = cfg.reuse
		var reuseCfg = reuse ? this.stack.select('id', reuse) : null
		var finalCfg

		if (reuseCfg) {
			finalCfg = ff.apply(reuseCfg, cfg)
		}
		else {
			cfg.id = this.seed++
			this.stack.push(cfg)
			finalCfg = cfg
		}

		if (finalCfg.timer) {
			finalCfg.timer.close()
		}

		if (finalCfg.timeout) {
			finalCfg.timer = new ff.Timer(finalCfg.timeout, true, function () {
				this.hide(finalCfg.id)
			}, this)
		}

		return finalCfg.id
	},


	hide: function (id) {
		if (id) {
			var cfg = this.stack.select('id', id)
			if (cfg) {
				this.stack.remove(cfg)
			}
		}
		else {
			this.stack.empty()
		}
	},


	info: ff.flex('[number] string [object] [number]', function (reuse, msg, args, timeout) {
		return this.show({
			cls: 'notify-info',
			msg: msg,
			args: args,
			reuse: reuse,
			timeout: timeout
		})
	}),


	alert: ff.flex('[number] string [object] [number]', function (reuse, msg, args, timeout) {
		return this.show({
			cls: 'notify-alert',
			msg: msg,
			args: args,
			reuse: reuse,
			timeout: timeout
		})
	}),


	success: ff.flex('[number] string [object] [number]', function (reuse, msg, args, timeout) {
		return this.show({
			cls: 'notify-success',
			msg: msg,
			args: args,
			reuse: reuse,
			timeout: timeout,
		})
	}),


	error: ff.flex('[number] string [object] [number]', function (reuse, msg, args, timeout) {
		return this.show({
			cls: 'notify-error',
			msg: msg,
			args: args,
			reuse: reuse,
			timeout: timeout
		})
	}),
}



Vue.component('setting-win', Vue.component('win').extend({

	data: {
		defaultConfig: null,
		config: {},
		dirty: false
	},

	methods: {
		save: function () {
			this.hidden = true
			ff.apply(this.defaultConfig, this.config)
		},


		cancel: function () {
			this.hidden = true
		},
	},

	watch: {
		hidden: function(value) {
			if (!value) {
				this.config = ff.deeplyClone(this.defaultConfig) || {}
				this.dirty = false

				var unWatchConfig = this.$watch('config', function () {
					this.dirty = true
					unWatchConfig()
				}, true)
			}
		}
	}
}))



Vue.component('video-win', Vue.component('win').extend({

	data: {
		url: '',

		title: 'Videos',

		width: 0,

		height: 0,

		hidden: false,

		bestWidth: 'auto',

		bestHeight: 'auto',

		isDisturbed: true,

		autoDestroy: true,
	},


	el: function () {
		return document.create([
		'<win class="video-win" f-el="win" f-style="width: bestWidth > 0 ? bestWidth + \'px\' : \'auto\'"',
			'f-class="disturbed: isDisturbed"',
			'f-on="mousemove: onMouseMove">',
			'<whead>{{title | trans}}</whead>',
			'<wbody>',
				'<video autoplay controls controlslist="nodownload" f-el="video" f-attr="src: url, width: bestWidth, height: bestHeight">',
			'</wbody>',
		'</win>'].join('\n'))
	},


	compiled: function () {
		this.$$.video.once('loadedmetadata', function(e) {
			var width = e.target.videoWidth
			var height = e.target.videoHeight
			if (width && height && (width !== this.width || height !== this.height)) {
				this.width  = width
				this.height = height
				this.onWindowResize()
			}
		}, this)
	},


	//we just want to double trigger play to make it show a loading message
	ready: function () {
		this.$$.video.play()
	},


	destroyed: function () {
		window.off('resize', this.onWindowResize, this)
		this.disturbTimer.close()
	},


	watch: {
		url: function () {
			this.$$.video.play()
		}
	},


	methods: {
		onWindowResize: function () {
			this.bestSize()
			Vue.flush()
			
			this.center()

			this.$el.off('show', this.center, this)
			this.$el.once('show', this.center, this)
		},


		bestSize: function () {
			if (!this.width || !this.height) {
				this.bestWidth  = 'auto'
				this.bestHeight = 'auto'
			}
			else {
				var w = Math.min(this.width,  document.documentElement.clientWidth  - 40),
					h = Math.min(this.height, document.documentElement.clientHeight - 40)

				if (w < this.width && w * this.height < h * this.width ) {
					this.bestWidth = w
					this.bestHeight = Math.round(this.height * w / this.width)
				}

				else if (h < this.height && w * this.height > h * this.width) {
					this.bestWidth = Math.round(this.width * h / this.height)
					this.bestHeight = h
				}

				else {
					this.bestWidth = w
					this.bestHeight = h
				}
			}
		},


		play: function () {
			this.$$.video.play()
		},


		pause: function () {
			this.$$.video.pause()
		},


		onMouseMove: function (e) {
			this.isDisturbed = true

			this.disturbTimer = this.disturbTimer || new ff.Timer(3000, function () {
				this.isDisturbed = false
			}, this)

			this.disturbTimer.reset()
		}
	}
}))




// Vue.component('video-frame', Vue.component('win').extend({

// 	data: {
// 		url: '',

// 		title: 'Videos',

// 		width: 0,

// 		height: 0,

// 		hidden: false,

// 		bestWidth: 'auto',

// 		bestHeight: 'auto',

// 		isDisturbed: true,

// 		autoDestroy: true,
// 	},


// 	el: function () {
// 		return document.create(['<win class="video-win video-frame" f-el="win" f-style="width: bestWidth > 0 ? bestWidth + \'px\' : \'auto\'"',
// 			'f-class="disturbed: isDisturbed"',
// 			'f-on="mousemove: onMouseMove">',
// 			'<whead>{{title | trans}}</whead>',
// 			'<wbody>',
// 				'<iframe f-attr="src: url, width: bestWidth, height: bestHeight" allowfullscreen></iframe>',
// 			'</wbody>',
// 		'</win>'].join('\n'))
// 	},


// 	destroyed: function () {
// 		window.off('resize', this.onWindowResize, this)
// 		this.disturbTimer.close()
// 	},


// 	methods: {
// 		onWindowResize: function () {
// 			this.bestSize()
// 			Vue.flush()

// 			this.$el.off('show', this.center, this)
// 			this.$el.once('show', this.center, this)
// 		},


// 		bestSize: function () {
// 			if (!this.width || !this.height) {
// 				this.bestWidth  = 'auto'
// 				this.bestHeight = 'auto'
// 			}
// 			else {
// 				var w = Math.min(this.width,  document.documentElement.clientWidth  - 40),
// 					h = Math.min(this.height, document.documentElement.clientHeight - 40)

// 				if (w < this.width && w * this.height < h * this.width ) {
// 					this.bestWidth = w
// 					this.bestHeight = Math.round(this.height * w / this.width)
// 				}

// 				else if (h < this.height && w * this.height > h * this.width) {
// 					this.bestWidth = Math.round(this.width * h / this.height)
// 					this.bestHeight = h
// 				}

// 				else {
// 					this.bestWidth = w
// 					this.bestHeight = h
// 				}
// 			}
// 		},


// 		onMouseMove: function (e) {
// 			this.isDisturbed = true

// 			this.disturbTimer = this.disturbTimer || new ff.Timer(3000, function () {
// 				this.isDisturbed = false
// 			}, this)

// 			this.disturbTimer.reset()
// 		}
// 	}
// }))



Vue.directive('play', {

	isLiteral: true,

	width: 640,

	height: 480,

	title: 'Videos',

	bind: function () {
		var el = this.el
		this.width  = Number(el.getAttribute('video-width'))  || this.width
		this.height = Number(el.getAttribute('video-height')) || this.height
		this.title  = el.getAttribute('video-title') || this.title

		el.removeAttribute('video-width')
		el.removeAttribute('video-height')
		el.removeAttribute('video-title')

		el.on('click', this.onClick, this, true)
	},


	onClick: function (e) {
		e.stopPropagation()

		var vm = new (Vue.component('video-frame'))({
			data: {
				url: this.raw,
				width: this.width,
				height: this.height,
				title: this.title,
			}
		})

		vm.$appendTo(document.body)
	}
})



Vue.component('audio-player', {

	replace: true,

	data: {
		list: null,

		playing: null,

		paused: true,

		src: '',

		volume: 1,

		autoplay: false,

		playedTime: 0,

		duration: 0,

		playMode: 'order',

		playModes: ['order', 'only', 'random'],
	},

	props: ['src-key', 'name-key', 'cover-key'],

	template: ['<div class="audio-player">',
		'<div class="audio-player-next" f-on="click: next"></div>',
		'<div class="audio-player-play" f-class="paused: paused" f-on="click: toggle"></div>',
		'<div class="audio-player-prev" f-on="click: prev"></div>',

		'<div class="audio-player-mode" f-class="playMode"',
			'f-on="click: togglePlayMode"',
			'f-tip="\'audio_mode_\' + playMode | trans" margin="10"></div>',

		'<div class="audio-player-cover" f-class="rotating: !paused">',
			'<div class="audio-player-img" f-lazysrc="cover" f-class="loaded: cover"></div>',
		'</div>',

		'<div class="audio-player-name">{{name || \'-\'}}</div>',
		'<div class="audio-player-time">{{playedTime | duration}}</div>',

		'<slider class="white audio-player-slider"',
			'f-with="value: playedTime, max: duration"',
			'f-events="change: onChangeTime">',
		'</slider>',

		'<div class="audio-player-duration">{{duration | duration}}</div>',

		'<div class="audio-player-volume"',
			'f-class="audio-player-mute: volume === 0"',
			'f-on="click: changeToMute"></div>',

		'<slider class="white audio-player-volume-slider"',
			'f-with="value: volume" max="{{1}}" step="{{0.01}}"',
			'f-events="change: onChangeVolume">',
		'</slider>',

		'<audio f-attr="src: src" f-attr="autoplay: autoplay" f-el="audio" preload="auto"></audio>',
	'</div>'].join('\n'),

	computed: {

		src: function () {
			var item = this.playing,
				key = this.srcKey

			return ff.get(item, key) || ''
		},


		name: function () {
			var item = this.playing,
				key = this.nameKey

			return ff.get(item, key) || ''
		},


		cover: function () {
			var item = this.playing,
				key = this.coverKey

			return ff.get(item, key) || ''
		}
	},


	ready: function () {
		this.$$.audio.on('timeupdate', function (e) {
			this.playedTime = this.$$.audio.currentTime
		}, this)

		this.$$.audio.on('volumechange', function (e) {
			this.volume = this.$$.audio.volume
		}, this)

		this.$$.audio.on('durationchange', function (e) {
			this.duration = this.$$.audio.duration
		}, this)

		this.$$.audio.on('play', function (e) {
			this.paused = false
		}, this)

		this.$$.audio.on('pause', function (e) {
			this.paused = true
		}, this)

		this.$$.audio.on('ended', this.onPlayEnd, this)

		this.$$.audio.volume = 1

		this.$el.on('show', function () {
			document.on('keydown', this.onKeyDown, this)
		}, this)

		this.$el.on('hide', function () {
			document.off('keydown', this.onKeyDown, this)
		}, this)

		document.body.append(this.$$.audio)
	},


	watch: {
		paused: function (paused) {
			if (this.paused) {
				this.$$.audio.pause()
			}
			else {
				this.$$.audio.play()
			}
		},

		//make audio autoplay when toggle src and playing music
		src: function (src) {
			if (src && !this.paused) {
				this.$$.audio.play()
				this.playedTime = 0
			}
		},


		list: function (list) {
			if (list && list.length > 0 && !this.playing) {
				this.playing = list[0]
			}
			else if (this.playing && list && !list.contains(this.playing)) {
				var playing = this.playing,
					srcKey = this.srcKey,
					src = ff.get(playing, srcKey)

				var newPlaying = list.find(function (item) {
					return ff.get(item, srcKey) === src
				})

				if (newPlaying) {
					this.playing = newPlaying
				}
			}
		}
	},


	methods: {

		toggle: function () {
			if (this.src) {
				this.paused = !this.paused
			}
		},


		prev: function () {
			var list = this.list,
				len = list ? list.length : 0,
				index = list.indexOf(this.playing) || 0

			this.playing = list[(index - 1).mod(len)]
			this.paused = false
			this.autoplay = true
		},


		next: function () {
			var list = this.list,
				len = list ? list.length : 0,
				index = list.indexOf(this.playing) || 0

			this.playing = list[(index + 1).mod(len)]
			this.paused = false
			this.autoplay = true
		},


		onPlayEnd: function () {
			var playMode = this.playMode

			if (playMode === 'only') {
				this.paused = false
			}
			else if (playMode === 'order') {
				this.next()
			}
			else {
				var list = this.list,
				len = list ? list.length : 0

				this.playing = list[Number.random(len)]
				this.paused = false
			}
		},


		onChangeTime: function (time) {
			this.$$.audio.currentTime = time
		},


		onChangeVolume: function (volume) {
			this.$$.audio.volume = volume
		},


		changeToMute: function () {
			if (this.volume === 0) {
				this.$$.audio.volume = this.lastVolume || 1
			}
			else {
				this.lastVolume = this.volume
				this.$$.audio.volume = 0
			}
		},


		togglePlayMode: function () {
			var index = (this.playModes.indexOf(this.playMode) + 1) % this.playModes.length
			this.playMode = this.playModes[index]
		},


		onKeyDown: function (e) {
			if (e.keyCode === 32) {
				this.paused = !this.paused
			}
		}
	}
})



Vue.component('tab', {

	replace: true,

	data: {
		activeIndex: 0
	},

	template: ['<div class="tab">',
		'<ul class="tab-btns" f-el="btns"><content select="btns" replace></content></ul>',
		'<div class="tab-panels" f-el="panels"><content select="panels" replace></content></div>',
	'</div>'].join('\n'),

	ready: function () {
		var btns = this.$$.btns,
			activedBtn = this.$$.btns.query('.actived')

		this.activeIndex = activedBtn ? activedBtn.elementIndex : 0

		btns.on('click', this.onClickBtn, this)
	},

	methods: {
		onClickBtn: function (e) {
			var btn = e.target.up('.tab-btns > *')

			if (btn) {
				this.activeIndex = btn.elementIndex
			}
		}
	},

	watch: {
		activeIndex: function(value, old) {
			var btns = this.$$.btns,
				panels = this.$$.panels

			if (old >= 0) {
				btns.children[old].removeClass('active')
				panels.children[old].hide()
			}

			if (value >= 0) {
				btns.children[value].addClass('active')
				panels.children[value].hidden = false
			}
		}
	}
})



ff.ThumbQueue = ff.extend(ff.Queue, {

	concurrency : 5,

	scrolling : 'down',	//up or down, if is down, load the thumbs after visible, else load thumbs before visible

	firstIndex : 0,

	lastIndex : 0,

	stateKey: 'thumbState',	//0(not load), 1(loading), 2(loaded)

	thumbKey: '',

	timeout: 0,	//use preload to control timeout

	preloadTimeout: 30000,

	preloadRetryTimes: 1,

	init: function () {
		ff.Queue.apply(this, arguments)
		this.onnext = this.onNextThumb.bind(this)
	},


	onNextThumb: function (task, next) {
		var stateKey = this.stateKey,
			thumbKey = this.thumbKey,
			url = ff.get(task, thumbKey, this.scope, task)

		task[stateKey] = 1
		
		ff.preload(url, this.preloadTimeout, this.preloadRetryTimes, function (img) {
			task[stateKey] = img ? 2 : 0
			next()
		})
	},


	//will immediatelly call updateIndex, the should call process
	updateTasks : function(tasks, startIndex) {
		var unloadedTasks = [],
			key = this.stateKey

		for (var i = 0, il = tasks.length; i < il; i++) {
			var task = tasks[i]

			if (!task[key]) {
				task.$add(key, 0)
				unloadedTasks.push(task)
				ff.defineValue(task, '__index', startIndex + i)
			}
		}

		this.tasks = unloadedTasks
		this.process()
	},


	updateVisibleIndexs: function (firstIndex, lastIndex) {
		if (firstIndex === this.firstIndex || lastIndex === this.lastIndex) {
			return
		}

		this.scrolling =  firstIndex > this.firstIndex || lastIndex > this.lastIndex ? 'down' : 'up'
		this.firstIndex = firstIndex
		this.lastIndex  = lastIndex


		var tasks = this.tasks

		var before   = [],
			newTasks = [],
			after    = []

		for (var i = 0; i < tasks.length; i++) {
			var task = tasks[i],
				index = task.__index

			if (index < firstIndex) {
				before.unshift(task)
			}
			else if (index >= lastIndex) {
				after.push(task)
			}
			else{
				newTasks.push(task)
			}
		}

		if (this.scrolling === 'down') {
			newTasks.pushAll(after)
			newTasks.pushAll(before)
		}
		else{
			newTasks.pushAll(before)
			newTasks.pushAll(after)
		}

		this.tasks = newTasks
	}
})



//items should use float or block layout
//content must be like:
//	item...
Vue.component('live', {

	data: {
		data: null,

		dest: null,	//the final data that has the same order with UI, after sort other action

		startBlockHeight: 0,

		endBlockHeight: 0,

		liveData: null,

		itemWidth: 'auto',	//include margin

		itemHeight: 0,

		cellCount: 1,

		startIndex: 0,	//items render start index

		endIndex: 0,	//items render end index

		grouping: 50,	//load items by group, one or more groups may exists at same time, the banlance point of item count and item render frequency

		preloadScreen: 0.5,	//preload how many screens, must larger than 0

		//when set, add a new property state in each item
		thumbKey: '',

		thumbPaused: false,

		visibleStartIndex: 0,

		visibleEndIndex: 0,
	},

	replace: true,

	template: ['<div class="live" f-el="body">',
		'<div class="live-block-start" f-style="height: startBlockHeight + \'px\'"></div>',
		'<content></content>',
		'<div class="live-block-end" f-style="height: endBlockHeight + \'px\'"></div>',
	'</div>'].join('\n'),

	props: ['data', 'grouping', 'preload-screen', 'thumb-key', 'thumb-paused'],


	ready: function () {
		var scroller = this.$$.body

		while (scroller && scroller.css) {
			var overflow = scroller.css('overflow-y')
			if (overflow === 'auto' || overflow === 'scroll') {
				break
			}
			scroller = scroller.parentNode
		}

		if (!scroller) {
			throw 'live component requires a scroller.'
		}

		this.$$.scroller = scroller
		this.$$.scroller.on('scroll', this.onScroll.block(50, true), this)
		window.on('resize', this.onWindowResize, this)

		if (this.thumbKey) {
			this.thumbQueue = new ff.ThumbQueue({
				thumbKey: this.thumbKey
			})

			this.$watch('thumbPaused', function (paused) {
				this.$el.visible() && !paused ? this.thumbQueue.resume() : this.thumbQueue.pause()
			})
		}
	},


	destroyed: function () {
		window.off('resize', this.onWindowResize, this)
	},


	methods: {
		measure: function () {
			var first = this.$$.body.firstElementChild.nextElementSibling
			if (first) {
				this.itemWidth  = first.outerWidth
				this.itemHeight = first.outerHeight
				this.cellCount = this.getCells()
			}
		},


		getCells: function () {
			if (!this.itemWidth || this.itemWidth === 'auto') {
				return 1
			}
			else{
				var cells = this.$$.scroller.clientWidth / this.itemWidth
				cells = Math.floor(cells)
				cells = Math.max(cells, 1)
				return cells
			}
		},


		getStartBlockHeight: function () {
			return Math.ceil(this.startIndex / this.cellCount) * this.itemHeight 
		},


		getEndBlockHeight: function () {
			if (!this.dest) {
				return 0
			}

			return Math.ceil((this.dest.length - this.endIndex) / this.cellCount) * this.itemHeight 
		},


		getLiveData: function () {
			var dest = this.dest

			if (dest) {
				return dest.slice(this.startIndex, this.endIndex)
			}
			else {
				return []
			}
		},


		getIndex: function (index) {
			return this.startIndex + index
		},


		onDataChanged: function (data, oldData) {
			if (data !== oldData) {
				this.$$.scroller.scrollTop = 0
			}

			this.dest = data

			//when data cleared, we need to render
			if (this.$el.visible() || !data) {
				this.render()
			}
			else if (data) {
				this.$el.off('show', this.render, this)
				this.$el.once('show', this.render, this)
			}
		},


		render: function () {
			if (this.dest && this.dest.length > 0 && !this.itemHeight) {
				this.liveData = this.dest.slice(0, 1)
				Vue.flush()
				this.measure()
			}

			this.onScroll(null, true)
		},


		getIndexByPosition: function (pos, direction, useGrouping) {
			var grouping = this.grouping,
				cells    = this.cellCount,
				height   = this.itemHeight,
				round    = direction === 'up' ? Math.floor : Math.ceil,
				length   = this.dest ? this.dest.length : 0

			var index = round(pos / this.itemHeight) * cells

			if (useGrouping) {
				index = round(index / grouping) * grouping
				index = round(index / cells) * cells
			}

			index = index.constrain(0, length)

			return index
		},


		onScroll: function (e, force) {
			var top      = this.$$.scroller.scrollTop,
				avHeight = this.$$.scroller.clientHeight,
				bot      = top + avHeight,
				preTop   = top - this.preloadScreen * avHeight,
				preBot   = bot + this.preloadScreen * avHeight

			var start        = this.getIndexByPosition(top, 'up', true),
				end          = this.getIndexByPosition(bot, 'down', true),
				visibleStart = this.getIndexByPosition(preTop, 'up'),
				visibleEnd   = this.getIndexByPosition(preBot, 'down')
			
			if (force || start !== this.startIndex || end !== this.endIndex) {
				this.startIndex = start
				this.endIndex  = end
				this.liveData = this.getLiveData()
				this.startBlockHeight = this.getStartBlockHeight()
				this.endBlockHeight = this.getEndBlockHeight()

				if (this.thumbQueue) {
					this.thumbQueue.updateTasks(this.liveData, this.startIndex)
				}
			}

			this.visibleStartIndex = visibleStart
			this.visibleEndIndex   = visibleEnd

			if (this.thumbQueue) {
				this.thumbQueue.updateVisibleIndexs(visibleStart, visibleEnd)
			}
		},


		onWindowResize: function () {
			this.$el.off('show', this.onVisibleAndResized, this)
			this.$el.once('show', this.onVisibleAndResized, this)
		},


		onVisibleAndResized: function () {
			var cells = this.getCells()

			if (cells !== this.cellCount) {
				this.cellCount = cells
				this.render()
			}
		}
	},


	watch: {
		thumbPaused: function (paused) {
			if (this.thumbQueue) {
				paused ? this.thumbQueue.pause() : this.thumbQueue.resume()
			}
		},


		data: function (data, oldData) {
			this.onDataChanged(data, oldData)
		},
	}
})



//items of each group should use float layout
//content must be like:
//	group
//		header
//		item contain
//			item...
//	group...
//	remember, never use sortBy or orderBy for liveData
Vue.component('glive', Vue.component('live').extend({

	data: {
		headerHeight: 0,

		groupData: null,

		groups: null,

		groupStart: 0,

		groupEnd: 0,

		visibleGroupStartIndex: 0,

		visibleGroupEndIndex: 0,

		visibleGroupKey: '',

		_visibleGroupKey: '',

		bottomSpace: false,

		groupKey: '',	//data can be array(use this key to group by) or grouped object

		reverseOrder: false,

		marginTop: 0,
	},


	props: ['group-key', 'bottom-space', 'reverse-order', 'margin-top'],


	methods: {

		measure: function () {
			var firstGroup = this.$$.body.firstElementChild.nextElementSibling
			if (firstGroup) {
				var firstItem = firstGroup.lastElementChild.firstElementChild
				this.itemWidth  = firstItem.outerWidth
				this.itemHeight = firstItem.outerHeight
				this.headerHeight = firstGroup.outerHeight - firstItem.outerHeight
				this.cellCount = this.getCells()
			}
		},


		getStartBlockHeight: function () {
			var groups = this.groups

			if (!groups || !groups.length) {
				return 0
			}

			var group  = groups[this.groupStart],
				start  = this.startIndex,
				height = this.itemHeight

			return group.startHeight
				+ Math.ceil((start - group.start) / this.cellCount) * height
		},


		getEndBlockHeight: function () {
			var groups = this.groups

			if (!groups || !groups.length) {
				return 0
			}

			var group  = groups[this.groupEnd],
				end    = this.endIndex,
				height = this.itemHeight,
				hdHeight = this.headerHeight,
				totalHeight = groups.last().endHeight,
				bottomSpace = this.bottomSpace

			var endHeight = group.startHeight
				+ hdHeight
				+ Math.ceil((end - group.start) / this.cellCount) * this.itemHeight

			if (bottomSpace) {
				var last = groups.last(),
					lastHeight = last.endHeight - last.startHeight,
					scrollerHeight = this.$$.scroller.clientHeight - this.$$.scroller.css('padding-top') - this.$$.scroller.css('padding-bottom')

				if (lastHeight < scrollerHeight) {
					totalHeight += scrollerHeight - lastHeight - this.marginTop
				}
			}

			return totalHeight - endHeight
		},


		getIndex: function (groupIndex, index) {
			var groupStart = this.groupStart,
				start = this.startIndex
			
			//as first group of live data
			if (groupIndex === 0) {
				return start + index
			}
			//not first group
			else {
				return this.groups[groupStart + groupIndex].start + index
			}
		},


		getLiveData: function () {
			var groups = this.groups

			if (!groups || !groups.length) {
				return {}
			}
			else {
				var groupStart = this.groupStart,
					groupEnd   = this.groupEnd,
					start      = this.startIndex,
					end        = this.endIndex

				var groupParts = groups.slice(groupStart, groupEnd + 1),
					keys = groupParts.pluck('key'),
					liveData = groupParts.indexValueBy('key', 'data'),
					groupInnerStart = start - groups[groupStart].start,
					groupInnerEnd   = end   - groups[groupEnd].start
				
				if (keys.length === 1) {
					var key = keys[0]
					liveData[key] = liveData[key].slice(groupInnerStart, groupInnerEnd)
				}
				else {
					var firstKey = keys[0],
						lastKey  = keys.last()

					liveData[firstKey] = liveData[firstKey].slice(groupInnerStart)
					liveData[lastKey ] = liveData[lastKey ].slice(0, groupInnerEnd)
				}

				return liveData
			}
		},


		render: function () {
			var data = this.data,
				groupKey = this.groupKey

			if (data instanceof Array && groupKey) {
				data = data.groupBy(groupKey)
			}

			this.groupData = data = data || {}
			this.dest = ff.values(data).flatten()

			var keys = Object.keys(data).sortBy(this.reverseOrder)

			if (keys.length > 0 && !this.itemHeight) {
				var firstKey = keys[0],
					liveData = {}

				liveData[firstKey] = data[firstKey].slice(0, 1)
				this.liveData = liveData
				Vue.flush()
				this.measure()
			}

			var totalHeight = 0,
				total       = 0,
				cells       = this.cellCount,
				height      = this.itemHeight,
				hdHeight    = this.headerHeight

			this.groups = ff.map(keys, function(key) {
				var length = data[key].length

				return {
					startHeight: totalHeight,
					endHeight  : totalHeight += Math.ceil(length / cells) * height + hdHeight,
					start      : total,
					end	       : total += length,
					key        : key,
					data       : data[key]
				}
			})

			this.onScroll(null, true)
		},


		getIndexByPosition: function (pos, direction, useGrouping) {
			var grouping = this.grouping,
				cells    = this.cellCount,
				height   = this.itemHeight,
				round    = direction === 'up' ? Math.floor : Math.ceil,
				groups   = this.groups,
				length   = groups ? groups.length : 0,
				hdHeight = this.headerHeight

			if (length === 0) {
				return {
					groupIndex: 0,
					index: 0
				}
			}

			var groupIndex = (pos === 'up' ? this.groupStart : this.groupEnd).constrain(0, length - 1)

			if (direction === 'up') {
				while (pos < groups[groupIndex].startHeight && groupIndex > 0) {
					groupIndex -= 1
				}

				while (pos > groups[groupIndex].endHeight && groupIndex < length - 1) {
					groupIndex += 1
				}
			}
			else {
				while (groupIndex > 0 && pos < groups[groupIndex].startHeight) {
					groupIndex -= 1
				}

				while (groupIndex < length - 1 && pos > groups[groupIndex].endHeight) {
					groupIndex += 1
				}
			}

			//index in all items
			var index = round((pos - hdHeight - groups[groupIndex].startHeight) / height) * cells + groups[groupIndex].start

			if (useGrouping) {
				index = round(index / grouping) * grouping

				if (direction === 'up') {
					while (groups[groupIndex].start > index && groupIndex > 0) {
						groupIndex -= 1
					}
				}
				else {
					while (groups[groupIndex].end < index && groupIndex < length - 1) {
						groupIndex += 1
					}
				}
			}

			var minIndex = groups[groupIndex].start,
				maxIndex = groups[groupIndex].end

			index = minIndex + round((index - minIndex) / cells) * cells
			index = index.constrain(minIndex, maxIndex)

			return {
				groupIndex: groupIndex,
				index: index
			}
		},


		onScroll: function (e, force) {
			var top      = this.$$.scroller.scrollTop,
				avHeight = this.$$.scroller.clientHeight,
				bot      = top + avHeight,
				preTop   = top - this.preloadScreen * avHeight,
				preBot   = bot + this.preloadScreen * avHeight

			var startO        	       = this.getIndexByPosition(preTop, 'up', true),
				endO          	       = this.getIndexByPosition(preBot, 'down', true),
				start         	       = startO.index,
				end           	       = endO.index,
				groupStart    	       = startO.groupIndex,
				groupEnd      	       = endO.groupIndex,
				visibleStartO 	       = this.getIndexByPosition(top, 'up'),
				visibleEndO            = this.getIndexByPosition(bot, 'down'),
				visibleStart           = visibleStartO.index,
				visibleEnd             = visibleEndO.index,
				visibleGroupStartIndex = visibleStartO.groupIndex,
				visibleGroupEndIndex   = visibleEndO.groupIndex

			if (force || start !== this.startIndex || end !== this.endIndex) {
				this.startIndex       = start
				this.endIndex         = end
				this.groupStart       = groupStart
				this.groupEnd         = groupEnd
				this.liveData         = this.getLiveData()
				this.startBlockHeight = this.getStartBlockHeight()
				this.endBlockHeight   = this.getEndBlockHeight()

				if (this.thumbQueue) {
					this.thumbQueue.updateTasks(ff.flatten(this.liveData), start)
				}
			}

			this.visibleStartIndex      = visibleStart
			this.visibleEndIndex        = visibleEnd
			this.visibleGroupStartIndex = visibleGroupStartIndex
			this.visibleGroupEndIndex   = visibleGroupEndIndex

			//this.$lock('visibleGroupKey')
			this.visibleGroupKey = this._visibleGroupKey
				= this.groups[visibleGroupStartIndex] ? this.groups[visibleGroupStartIndex].key : ''
			
			if (this.thumbQueue) {
				this.thumbQueue.updateVisibleIndexs(visibleStart, visibleEnd)
			}
		},


		onWindowResize: function () {
			Vue.component('live').options.methods.onWindowResize.apply(this, arguments)
			this.endBlockHeight = this.getEndBlockHeight()
		}
	},


	watch: {
		visibleGroupKey: function (key) {
			if (key !== this._visibleGroupKey) {
				var groups = this.groups,
					group = groups && key ? groups.select('key', key) : null

				if (group) {
					this.$$.scroller.scrollTop = group.startHeight
					this.onScroll(null)
				}
			}
		}
	}
}))



Vue.component('grid', Vue.component('live').extend({

	replace: true,

	data: {
		sortKey: '',

		sortOrder: 0,
	},

	template: ['<div class="grid">',
		'<div class="grid-head" f-el="head">',
			'<content select="ghead" replace></content>',
		'</div>',
		'<div class="live grid-body" f-el="body">',
			'<div class="live-block-start" f-style="height: startBlockHeight + \'px\'"></div>',
			'<content select="gbody" replace></content>',
			'<div class="live-block-end" f-style="height: endBlockHeight + \'px\'"></div>',
		'</div>',
	'</div>'].join('\n'),


	beforeCompile: function () {
		var cols = ff.toArray(this.$el.query('.grid-head').children)
		if (cols.length > 0) {
			for (var i = 0; i < cols.length; i++) {
				var col = cols[i],
					key = col.attr('key')

				if (key) {
					col.removeAttribute('key')
					col.addClass('grid-sort')
					col.attr('f-on', 'click: sortByKey(\'{0}\')'.format(key))
					col.attr('f-class', 'getSortClass(\'{0}\')'.format(key))
					col.append('<div class="grid-sort-arrow"></div>'.format(key))
				}
			}
		}
	},


	computed: {
		hasScrollBar: function () {
			return this.dest ? this.dest.length * this.itemHeight > this.$$.scroller.clientHeight : false
		}
	},


	methods: {
		getSortClass: function (key) {
			if (key === this.sortKey) {
				if (this.sortOrder === 1) {
					return 'grid-sort-asc'
				}
				else if (this.sortOrder === -1) {
					return 'grid-sort-desc'
				}
			}

			return ''
		},


		sortByKey: function (key) {
			if (key === this.sortKey) {
				this.sortOrder = this.sortOrder === 1 ? -1 : 1
			}
			else if (this.sortKey) {
				this.sortKey = ''
				this.sortOrder = 0
			}
			else {
				this.sortKey = key
				this.sortOrder = 1
			}

			this.doSort()
		},


		doSort: function () {
			var key = this.sortKey

			if (key) {
				this.dest = ff.sortBy(this.dest, function (item) {
					var value = item[key]
					if (typeof value === 'number') {
						return value
					}
					else {
						return String(value).toLowerCase()
					}
				}, this.sortOrder === -1)
			}
			else {
				this.dest = this.data
			}
			
			this.onScroll(null, true)
		}
	},


	watch: {
		hasScrollBar: function (value) {
			if (value) {
				this.$$.head.css('padding-right', document.scrollBarWidth)
			}
			else {
				this.$$.head.css('padding-right', 0)
			}
		},

		data: function (data) {
			if (data) {
				this.doSort()
			}
		}
	}
}))



Vue.component('tree-leaf', {

	data: {
		data: null,
		
		root: null,

		menuExpand: false,

		loading: false,

		children: null,

		isSelected: false,

		isEmpty: false,

		visible: true,
	},


	template: ['<div class="tree-leaf"',
			'f-el="name"',
			'f-show="visible"',
			'f-class="selected: isSelected"',
			'f-on="click: onClickLeaf, dblclick: onDblClickLeaf">',
			'<span class="tree-icon" f-class="tree-expand: menuExpand, tree-loading: loading, tree-empty: isEmpty" f-on="click: onClickIcon"></span>',
			'<span class="tree-name" f-class="tree-folder: isFolder">{{name}}</span>',
		'</div>',
		'<ul class="tree-menu" f-show="menuExpand" f-transition="collapse">',
			'<li class="tree-item"',
				'f-component="tree-leaf"',
				'f-repeat="data: visibleChildren"',
				'f-with="root: root">',
			'</li>',
		'</ul>'].join('\n'),


	ready: function () {
		var path = this.path
		this.root.register(path, this)
	},


	computed: {
		name: function () {
			return ff.get(this.data, this.root.nameKey)
		},


		isFolder: function () {
			return ff.get(this.data, this.root.folderKey)
		},


		path: function () {
			return ff.get(this.data, this.root.pathKey)
		},


		visibleChildren: function () {
			if (this.children) {
				var folderKey = this.root.folderKey
				var vc = ff.filter(this.children, function (child) {
					return ff.get(child, folderKey)
				}, this)

				return vc
			}
			else {
				return this.children
			}
		},


		isEmpty: function () {
			if (this.children) {
				var folderKey = this.root.folderKey,
					children = this.children

				for (var i = 0; i < children.length; i++) {
					var child = children[i],
						isFolder = ff.get(child, folderKey)

					if (isFolder) {
						return false
					}
				}

				return true
			}
			else {
				return false
			}
		}
	},


	methods: {
		onClickLeaf: function () {
			if (this.root.currentPath !== this.path) {
				this.root.currentPath = this.path
			}
			else {
				this.expandMenu()
			}
		},


		onClickIcon: function (e) {
			if (e) {
				e.stopPropagation()
			}

			this.toggleMenu()
		},


		onDblClickLeaf: function () {
			if (this.menuExpand) {
				this.menuExpand = false
			}
			else {
				this.onClickLeaf()
			}
		},


		toggleMenu: function () {
			if (this.menuExpand) {
				this.menuExpand = false
			}
			else {
				this.expandMenu()
			}
		},


		expandMenu: function () {
			if (this.children) {
				this.menuExpand = true
			}
			else if (!this.loading) {
				this.loading = true
				ff.getAsync(this.data, this.root.childKey, (function (children) {
					this.children = children
					this.loading = false
					this.menuExpand = true
				}).wait(this.root.timeout, this))
			}
		}
	}
})



Vue.component('tree', Vue.component('tree-leaf').extend({

	replace: true,

	data: {
		menuExpand: true,

		root: null,

		isSelected: true,

		timeout: 30000,

		lineHeight: 30,

		pathKey: '',

		folderKey: '',

		nameKey: '',

		childKey: '',

		selectedLeaf: null,

		currentPath: '',

		leafs: {},

		//if is false, hide current tree node, but show children
		visible: false,
	},


	created: function () {
		this.root = this
	},


	props: ['data', 'path-key', 'folder-key', 'name-key', 'child-key', 'only-folder', 'timeout', 'line-height'],


	template: ['<div class="tree">',
		'<div class="tree-inner">',
			Vue.component('tree-leaf').options.template,
		'</div>',
	'</div>'].join('\n'),


	watch: {
		//change from outside, path must already registered
		currentPath: function (path) {
			var leaf = this.leafs[path]

			if (leaf) {
				this.select(leaf)
				leaf.expandMenu()
			}
		}
	},


	methods: {
		select: function (leaf) {
			if (this.selectedLeaf) {
				this.selectedLeaf.isSelected = false
			}

			leaf.isSelected = true
			leaf.$$.name.scrollToView('y', this.lineHeight)
			this.selectedLeaf = leaf
		},


		register: function (path, leaf) {
			this.leafs[path] = leaf
		}
	}
}))



//needs trans upload_uploading & upload_left
Vue.component('upload-pro', {

	replace: true,

	data: {
		uploader: null
	},

	props: ['uploader'],

	template: ['<div class="upload-pro" f-show="uploader.unprocessed > 0">',
		'<div class="upload-pro-inner"',
			'f-style="width: uploader.loaded / uploader.total * 100 + \'%\'"',
			'f-class="upload-pro-side: uploader.loaded === uploader.total || uploader.loaded === 0"',
		'></div>',
		'<div class="upload-pro-icon"></div>',
		'<div class="upload-pro-text">',
			'<span>',
				'{{$filter(\'trans\')(\'upload_uploading\', uploader.name)}}',
				'&emsp;( {{$filter(\'trans\')(\'upload_left\', uploader.unprocessed)}} )',
			'</span>',
		'</div>',
		'<div class="upload-pro-close" f-on="click: close"></div>',
	'</div>'].join('\n'),


	methods: {
		close: function () {
			this.uploader.end(true)
		}
	}
})



Vue.component('upload-cap', {
	replace: true,

	data: {
		over: false,

		capable: true,

		ondrop: null,
	},

	props: ['capable', 'ondrop'],

	template: '<div class="upload-cap" f-class="dragging: over" f-on="mouseup: onMouseUp"></div>',

	ready: function () {
		document.body.on('dragenter', function (e) {
			e.preventDefault()
			if (this.capable) {
				this.over = true
			}
		}, this)

		this.$el.on('dragover', function (e) {
			e.preventDefault()
		}, this)

		this.$el.on('dragleave', function (e) {
			e.preventDefault()
			this.over = false
		}, this)

		this.$el.on('drop', function (e) {
			e.preventDefault()
			this.over = false

			if (this.capable) {
				this.getFilesFromDataTransfer(e.dataTransfer, this.ondrop)
			}
		}, this)
	},


	methods: {
		onMouseUp: function (e) {
			this.over = false
		},


		getFilesFromDataTransfer: function (transfer, callback) {
			if (transfer.items && window.DataTransferItem && DataTransferItem.prototype.webkitGetAsEntry) {
				var transferFiles = Array.from(transfer.files)
				var files = []
				var items = ff.toArray(transfer.items).selectAll({kind: 'file'})

				var onerror = function () {
					callback.call(this, transferFiles)
				}.bind(this)

				var readFilesFromEntry = function (entry, callback, reader) {
					if (!entry) {
						callback()
					}
					else if (entry.isFile) {
						entry.file(function (file) {
							file._relativePath = entry.fullPath
							files.push(file)
							callback()
						}, onerror)
					}
					else if (entry.isDirectory) {
						var reader = reader || entry.createReader()
						reader.readEntries(function (entries) {
							if (entries && entries.length) {
								ff.eachAsync(ff.toArray(entries),
									function (entry, next) {
										readFilesFromEntry(entry, next)
									},
									function () {
										readFilesFromEntry(entry, callback, reader)
									}
								)
							}
							else {
								callback()
							}
						}, onerror)
					}
				}

				var completed = 0
				for (var i = 0; i < items.length; i++) {
					var item = items[i]
					var entry = item.webkitGetAsEntry()
					readFilesFromEntry(entry, function () {
						if (++completed >= items.length) {
							callback.call(this, files)
						}
					})
				}
			}
			else {
				callback.call(this, transferFiles)
			}
		}
	}
})




//support events:
//	show
//	hide
Vue.component('viewer', {

	replace: true,

	data: {
		//config
		data: [],

		srcKey : 'path',

		thumbKey: 'thumbPath',

		widthKey: 'width',	//must be string

		heightKey: 'height',

		nameKey: 'name',

		rotateKey: 'rotate',

		scaleMin : 0.1,

		scaleMax : 10,

		scaleDelta : 1.25,

		thumbPreview : 1,

		thumbPreload: 3,	//to prevent blank appears when running animation

		playIntervalTime : 5000,

		minPlayIntervalTime: 3000,

		maxPlayIntervalTime: 30000,

		isOriginalSize: false,	//default is false, image contained by body

		actions: null,

		setbgable: true,

		deletable: true,

		//always true, the Width and Height of item is always be fixed
		//isSizeFixedByRotation: true,

		//the state that image rotation
		isRotationClockwise: true,


		//properties
		thumbCount : 0,

		thumbArrowWidth: 40,

		thumbVisible: true,

		thumbIndex: 0,	//the first thumb that is visible

		isPlaying: false,

		isDisturbed: false,

		index : 0,

		direction : 1,

		dragX : 10,	//specify the offset to the center, without scaling

		dragY : 10,

		scale : 1,

		rotate : 0,

		hidden: false,

		bodyWidth: 0,

		bodyHeight: 0,

		thumbData: [],

		thumbDataStart: -1,	//make sure it can be different and refresh thumb data

		draging: false,

		switching: false,


		//other properties which will keep consistant with css
		thumbWidth: 64,

		thumbGap: 4,

		minThumbArrowWidth: 40,
	},


	props: ['src-key', 'thumb-key', 'width-key', 'height-key', 'name-key', 'rotate-key'],


	template: ['<div class="viewer" f-hide="hidden" f-transition="fade"',
		'f-class="playing: isPlaying, disturbed: isDisturbed, thumb-hidden: !thumbVisible">',
		'<div class="viewer-body" f-el="body"',
			'f-on="mousewheel: onMouseWheel">',
			'<div class="viewer-img-wrap" f-el="imgWrap" f-imgsrc="src"',
				'f-on="mousedown: onDragStart, load: onImgLoad"',
				'f-class="draging: draging"',
				'f-style="',
					'transform: \'translate3d({dragX}px, {dragY}px, 0) scale({scale}) rotate({rotate}deg)\'.format(this),',
					'width: imgWidth + \'px\',',
					'height: imgHeight + \'px\',',
					'left: imgLeft + \'px\',',
					'top: imgTop + \'px\'" >',
				'<div class="viewer-img-background" f-style="',
					'background-image: \'url(&quot;{0}&quot;)\'.format(getThumb(item)),',
					'transform: \'translate(-50%, -50%) rotate({imgRotation}deg)\'.format(this),',
					'width: fixedImgWidth + \'px\',',
					'height: fixedImgHeight + \'px\',',
				'"></div>',
				'<img class="viewer-img" f-imgsrc="src">',
			'</div>',
		'</div>',

		'<div class="viewer-close" f-on="click: hide"></div>',
		'<div class="viewer-prev" f-on="click: prev, mouseenter: onEnterBar, mouseleave: onLeaveBar"></div>',
		'<div class="viewer-next" f-on="click: next, mouseenter: onEnterBar, mouseleave: onLeaveBar"></div>',

		'<div class="viewer-thumbbar">',
			'<div class="viewer-thumbbar-prev" f-on="mousedown: thumbMouseDown(-1)" f-style="width: thumbArrowWidth + \'px\'" f-show="data.length > thumbCount"></div>',
			'<div class="viewer-thumbbar-body" f-style="width: thumbWidth * maxThumbCount + \'px\'"',
				'f-on="mousewheel: onThumbMouseWheel">',
				'<ul class="viewer-thumbbar-inner" f-el="thumbInner" f-style="width: thumbWidth * (data ? data.length : 0) + \'px\',',
						'left: -thumbWidth * thumbIndex + \'px\'">',
					'<div class="viewer-thumbbar-block" f-style="width: thumbWidth * thumbDataStart + \'px\'"></div>',
					'<li f-repeat="thumbData" f-class="active: $value === item"',
						'f-on="click: view(thumbDataStart + $index)">',
						'<div class="viewer-thumbbar-img" f-class="loaded: viewerThumbState === 2"',
							'f-style="background-image: viewerThumbState === 2 ? \'url(&quot;{0}&quot;)\'.format(getThumb($value)) : \'\',',
								'transform: getRotateDegree($value)',
								'"></div>',
					'</li>',
				'</ul>',
			'</div>',
			'<div class="viewer-thumbbar-next" f-on="mousedown: thumbMouseDown(1)" f-style="width: thumbArrowWidth + \'px\'" f-show="data.length > thumbCount"></div>',
		'</div>',

		'<div class="viewer-toolbar" f-on="mouseenter: onEnterBar, mouseleave: onLeaveBar">',
			'<div class="viewer-toolbar-info">',
				'<div class="viewer-toolbar-pixels">\{{imgWidth}} x {{imgHeight}}&nbsp;</div>',
				'<div class="viewer-toolbar-scale">\{{parseInt(scale * 100)}}%&nbsp;</div>',
				'<div class="viewer-toolbar-name">{{name}}</div>',
			'</div>',

			'<ul class="viewer-toolbar-actions">',
				'<li class="viewer-toolbar-zoomin" f-tip="\'Zoom in\' | trans" f-on="click: this.zoom(1)"></li>',
				'<li class="viewer-toolbar-zoomout" f-tip="\'Zoom out\' | trans" f-on="click: this.zoom(-1)"></li>',
				'<li class="viewer-toolbar-size-contain" f-tip="isOriginalSize ? \'original Size\' : \'Best Size\' | trans" f-on="click: changeSize" f-class="viewer-toolbar-size-original: isOriginalSize"></li>',
				'<li class="viewer-toolbar-setbg" f-tip="\'Set As Wallpaper\' | trans" f-on="click: setbg" f-show="setbgable"></li>',
				'<li class="viewer-toolbar-prev" f-tip="\'Previous\' | trans" f-on="click: prev"></li>',
				'<li class="viewer-toolbar-play" f-tip="isPlaying ? \'Stop\' : \'Play\' | trans" f-on="click: play" f-class="viewer-toolbar-stop: isPlaying"></li>',
				'<li class="viewer-toolbar-next" f-tip="\'Next\' | trans" f-on="click: next"></li>',
				'<li class="viewer-toolbar-rotateleft" f-tip="\'Rotate left\' | trans" f-on="click: setRotate(-1)"></li>',
				'<li class="viewer-toolbar-rotateright" f-tip="\'Rotate right\' | trans" f-on="click: setRotate(1)"></li>',
				'<li class="viewer-toolbar-download" f-tip="\'Download\' | trans" f-on="click: download"></li>',
				'<li class="viewer-toolbar-delete" f-tip="\'Delete\' | trans" f-on="click: delete" f-show="deletable"></li>',
			'</ul>',

			'<div class="viewer-toolbar-thumb-switch">',
				'{{\'thumbnail\' | trans}}',
				'<switch class="white" f-with="value: thumbVisible" f-events="change: toggleThumbVisibility">',
					'<div class="viewer-toolbar-toggle-thumb-ball"></div>',
				'</switch>',
			'</div>',

			'<slider class="viewer-toolbar-play-interval bigger"',
				'f-with="value: playIntervalTime" min="{{minPlayIntervalTime}}" max="{{maxPlayIntervalTime}}" step="{{1000}}"',
				'f-events="change: newPlaying">',
			'</slider>',
			'<div class="viewer-toolbar-play-interval-time">',
				'{{playIntervalTime / 1000}}s',
			'</div>',
			'<div class="viewer-toolbar-progress">',
				'{{index + 1}} / {{data.length}}',
			'</div>',
	'</div>'].join('\n'),


	created: function () {
		this.thumbVisible = ff.store.get('viewer_thumb_visible') !== false
	},


	ready: function () {
		this.onWindowResize = this.onWindowResize.block(200, true)
		window.on('resize', this.onWindowResize, this)
		this.onWindowResize()

		document.on('keydown', this.onKeyDown, this)

		var playIntervalTime = ff.store.get('viewer_play_interval_time')
		if (playIntervalTime && playIntervalTime >= this.minPlayIntervalTime && playIntervalTime <= this.maxPlayIntervalTime && playIntervalTime % 1000 === 0) {
			this.playIntervalTime = playIntervalTime
		}

		this.thumbQueue = new ff.ThumbQueue({
			thumbKey: this.thumbKey,
			stateKey: 'viewerThumbState'
		})

		this.$el.on('toggle', function (e) {
			e.state ? this.thumbQueue.resume() : this.thumbQueue.pause()
		}, this)
	},


	destroyed : function () {
		window.off('resize', this.onWindowResize, this)
		document.off('keydown', this.onKeyDown, this)
	},


	computed: {

		item: function () {
			return this.data[this.index]
		},


		name: function () {
			return ff.get(this.item, this.nameKey) || '-'
		},


		src: function () {
			return ff.get(this.item, this.srcKey) || ''
		},


		imgWidth: function () {
			var key = this.isOddRotation(this.item) ? this.heightKey : this.widthKey
			return ff.get(this.item, key) || Math.min(this.bodyWidth, this.bodyHeight)
		},


		imgHeight: function () {
			var key = this.isOddRotation(this.item) ? this.widthKey : this.heightKey
			return ff.get(this.item, key) || Math.min(this.bodyWidth, this.bodyHeight)
		},


		fixedImgWidth: function () {
			return ff.get(this.item, this.widthKey) || Math.min(this.bodyWidth, this.bodyHeight)
		},


		fixedImgHeight: function () {
			return ff.get(this.item, this.heightKey) || Math.min(this.bodyWidth, this.bodyHeight)
		},


		imgRotation: function () {
			return ff.get(this.item, this.rotateKey)
		},


		imgLeft: function () {
			return (this.bodyWidth - this.imgWidth) / 2
		},


		imgTop: function () {
			return (this.bodyHeight - this.imgHeight) / 2
		},


		maxThumbCount: function () {
			return Math.min(this.thumbCount, this.data.length)
		}
	},


	methods: {

		show: function () {
			this.hidden = false
			document.off('keydown', this.onKeyDown, this)
			document.on('keydown', this.onKeyDown, this)
		},


		hide: function () {
			this.hidden = true
			if (this.isPlaying) {
				this.play()
			}
			document.off('keydown', this.onKeyDown, this)
		},


		getThumb: function (item) {
			return ff.get(item, this.thumbKey) || ''
		},


		onWindowResize : function(e) {
			this.$el.off('show', this.adjust, this)
			this.$el.once('show', this.adjust, this)
		},


		getRotateDegree: function (item) {
			var degree = ff.get(item, this.rotateKey)

			if (!degree) {
				return ''
			}
			else {
				return 'rotate({0}deg)'.format(degree)
			}
		},


		isOddRotation: function (item) {
			var degree = ff.get(item, this.rotateKey)
			return degree % 180 === 90
		},


		//available size change to trigger
		adjust : function (onlyBody) {
			this.adjustBody()

			if (this.item) {
				this.dragX = this.dragY = 0
				this.adjustScale(true)
			}

			this.adjustThumbbar()
		},


		//when page resize, toggle playing mode, or toggle thumbbar visibility
		adjustBody: function () {
			this.bodyWidth = this.$$.body.offsetWidth,
			this.bodyHeight = this.$$.body.offsetHeight
		},


		adjustThumbbar : function () {
			var aw = this.$el.offsetWidth,
				tw = this.thumbWidth,
				gap = this.thumbGap,
				count = Math.floor((aw - gap - (this.minThumbArrowWidth + gap) * 2) / tw)

			this.thumbCount = count
			this.thumbArrowWidth = Math.min((aw - gap - count * tw) / 2 - gap, tw - gap)
		},


		view : function(index) {
			this.direction = index >= this.index ? 1 : -1
			this.index = index.mod(this.data.length)
			this.rotate = (this.isRotationClockwise ? -1 : 1) * ff.get(this.item, this.rotateKey) || 0
			this.adjustScale(false)

			if (this.hasNoSize(this.item)) {
				this.tryGetLoadedImgSize(this.item)
			}

			Vue.flush()

			this.setThumbIndex(this.thumbIndex, true)
			this.preloadNextItem()
		},


		hasNoSize: function (item) {
			return !ff.get(item, this.widthKey) || !ff.get(item, this.heightKey)
		},


		tryGetLoadedImgSize: function (item) {
			var url = ff.get(item, this.srcKey),
				img = document.create('<img src="{0}" hidden>'.format(url))

			document.body.append(img)

			this.applySizeByImg(item, img)
		},


		applySizeByImg: function (item, img) {
			if(!img) {
				return
			}
			
			var w = img.naturalWidth
			var h = img.naturalHeight
			var isChanged = false

			if (w && h) {
				if (this.isOddRotation(item)) {
					isChanged = h !== item[this.widthKey] || w !== item[this.heightKey]

					if (isChanged) {
						item[this.widthKey] = h
						item[this.heightKey] = w
					}
				}
				else {
					isChanged = w !== item[this.widthKey] || h !== item[this.heightKey]

					if (isChanged) {
						item[this.widthKey] = w
						item[this.heightKey] = h
					}
				}
			}

			return isChanged
		},


		onImgLoad: function (e) {
			var item = this.item,
				img = e.target,
				w = img.naturalWidth,
				h = img.naturalHeight

			if (this.applySizeByImg(item, img)) {
				this.adjustScale(true)
			}
		},


		preloadNextItem: function () {
			var nextItem = this.data[this.index + this.direction]
			if (nextItem) {
				var src = ff.get(nextItem, this.srcKey)
				ff.preload(src, function (img) {
					this.applySizeByImg(nextItem, img)
				}, this)
			}
		},


		//to cacl the right scale when image changed, or available size change
		//will reset dragX & drayY, scale, not will not reset rotate
		//when reset some properties for same img, useAnmi should be true
		adjustScale : function(isSameImage) {
			var oldScale = this.scale

			var item = this.item
			if (!item) {
				return
			}

			var ow = this.imgWidth,
				oh = this.imgHeight,
				maxw = this.bodyWidth,
				maxh = this.bodyHeight,
				isV = this.rotate.mod(180) === 90,
				w = isV ? oh : ow,
				h = isV ? ow : oh,
				scale = 1

			if (this.isOriginalSize) {
				scale = 1
			}
			else {
				if (w > maxw) {
					scale = maxw / w
				}

				if (h * scale > maxh) {
					scale = maxh / h
				}
			}
			
			this.dragX = this.dragY = 0
			this.scale = scale

			//disable animation when change to another image
			if (!isSameImage) {
				this.$$.imgWrap.css({
					transition: 'none'
				})

				Vue.flush()
				this.$$.imgWrap.redraw()
				this.$$.imgWrap.css('transition', '')
			}
		},


		prev : function () {
			this.view(this.index - 1)

			if (this.isPlaying) {
				this.newPlaying()
			}
		},


		next : function () {
			this.view(this.index + 1)

			if (this.isPlaying) {
				this.newPlaying()
			}
		},


		//set and adjust the thumb index
		setThumbIndex: function (thumbIndex, makesureIndexVisible) {
			var index = this.index,
				thumbCount = this.thumbCount,
				thumbPreview = this.thumbPreview,
				len = this.data.length,
				maxThumbCount = this.maxThumbCount

			if (makesureIndexVisible && thumbIndex > index - thumbPreview) {
				thumbIndex = index - thumbPreview
			}
			else if (makesureIndexVisible && thumbIndex + maxThumbCount < index + thumbPreview + 1) {
				thumbIndex = index + thumbPreview + 1 - maxThumbCount
			}

			thumbIndex = Math.max(thumbIndex, 0)
			thumbIndex = Math.min(thumbIndex, len - maxThumbCount)

			this.thumbIndex = thumbIndex
			this.compareThumbData()
		},


		//return the index that start to render thumb data
		compareThumbData: function (force) {
			var index = this.thumbIndex,
				thumbCount = this.thumbCount,
				preload = this.thumbPreload,
				len = this.data.length

			var start = Math.floor(index / thumbCount) * thumbCount,
				end = start + 2 * (thumbCount + preload)

			if (end >= len) {
				end = len
				start = end - 2 * thumbCount
				start = Math.max(start, 0)
			}

			if (force || start !== this.thumbDataStart) {
				this.thumbDataStart = start
				this.thumbData = this.data.slice(start, end)
				this.thumbQueue.updateTasks(this.thumbData, start)
			}

			this.thumbQueue.updateVisibleIndexs(index, Math.min(index + thumbCount, end))
		},


		thumbMouseDown: function (direction) {
			this.setThumbIndex(this.thumbIndex + direction)

			this.startTime = this.startTime || Date.now()

			this.$$.thumbInner.on('transitionend', (function () {
				this.setThumbIndex(this.thumbIndex + direction)
			}).block(50), this)

			this.$$.thumbInner.once('transitionend', function () {
				this.$$.thumbInner.addClass('holding')
			}, this)

			document.once('mouseup', function () {
				this.$$.thumbInner.off('transitionend')
				this.$$.thumbInner.removeClass('holding')
			}, this)
		},


		onThumbMouseWheel: function (e) {
			e.stopPropagation()
			e.preventDefault()

			if (e.wheelDelta > 0 || e.detail < 0) {
				this.setThumbIndex(this.thumbIndex - 1)
			}
			else{
				this.setThumbIndex(this.thumbIndex + 1)
			}
		},


		onKeyDown : function(e) {
			if (e.keyCode === 27) {
				this.hide()
			}
			else if (e.keyCode === 46) {
				if (this.isPlaying) {
					this.play()
				}
				this.delete()
			}
			else if (e.keyCode === 37 || e.keyCode === 38) {
				this.prev()
			}
			else if (e.keyCode === 39 || e.keyCode === 40) {
				this.next()
			}
			else if (e.keyCode === 32) {
				this.play()
			}
		},


		//here it will be triggered frequently
		onMouseWheel : function(e) {
			e.preventDefault()

			if (e.wheelDelta > 0 || e.detail < 0) {
				this.zoom(1, e)
			}
			else{
				this.zoom(-1, e)
			}
		},


		zoom: function (direction, e) {
			var oldScale = this.scale,
				newScale = this.scale * Math.pow(this.scaleDelta, direction)

			var oldPer = Math.round(oldScale * 100),
				newPer = Math.round(newScale * 100)

			if(oldPer < 100 && newPer > 100 || oldPer > 100 && newPer < 100 ){
				newScale = 1
			}
			else {
				newScale = newScale.constrain(this.scaleMin, this.scaleMax)
			}

			this.scale = newScale

			if (e) {
				var rate = newScale / oldScale,
					rel = [e.pageX, e.pageY],
					center = [this.bodyWidth / 2, this.bodyHeight / 2],
					oldDrag = [this.dragX, this.dragY],
					now = [center[0] + oldDrag[0], center[1] + oldDrag[1]],	//center + oldDrag
					dest = [rel[0] * (1 - rate) + now[0] * rate, rel[1] * (1 - rate) + now[1] * rate],	//rel * (1-rate) + now * rate
					newDrag = [dest[0] - center[0], dest[1] - center[1]]	//dest - center

				this.setDragXY(newDrag[0], newDrag[1])
			}
		},


		setDragXY: function (dragX, dragY) {
			var scale = this.scale,
				aw = this.bodyWidth,
				ah = this.bodyHeight,
				w  = this.imgWidth,
				h  = this.imgHeight,
				gapX = aw / 2 - w * scale / 2,
				gapY = ah / 2 - h * scale / 2

			if (gapX > 0) {
				dragX = 0
			}
			else {
				dragX = dragX.constrain(gapX, -gapX)
			}

			if (gapY > 0) {
				dragY = 0
			}
			else {
				dragY = dragY.constrain(gapY, -gapY)
			}

			this.dragX = dragX
			this.dragY = dragY
		},


		onDragStart : function(e) {
			e.stopPropagation()
			e.preventDefault()

			this.draging = true

			var startDragX = this.dragX,
				startDragY = this.dragY,
				startX = e.pageX,
				startY = e.pageY,
				scale = this.scale

			var onMouseMove = function (e) {
				this.dragX = startDragX + (e.pageX - startX)
				this.dragY = startDragY + (e.pageY - startY)
			}

			document.on('mousemove', onMouseMove, this)

			document.once('mouseup', function (e) {
				this.draging = false
				document.off('mousemove', onMouseMove, this)
				this.setDragXY(this.dragX, this.dragY)
			}, this)
		},


		changeSize : function () {
			this.isOriginalSize = !this.isOriginalSize
			this.adjustScale(true)
		},


		setRotate : function(factor) {
			this.rotate = this.rotate + 90 * factor
			this.adjustScale(true)
		},

		download : function () {
			this.actions.download(this.item)
		},


		setbg : function () {
			this.actions.setbg(this.item, this.rotate)
		},


		delete : function () {
			this.actions.delete(this.item)
		},


		toggleThumbVisibility: function () {
			Vue.flush()

			this.adjustBody()
			this.adjustScale(true)

			ff.store.set('viewer_thumb_visible', this.thumbVisible)
		},


		fullscreen : function () {
			var el = this.$el;
			(el.requestFullScreen || el.webkitRequestFullScreen || el.mozRequestFullScreen || el.msRequestFullScreen).call(el)
		},


		cancelFullScreen: function () {
			var el = document;
			(el.cancelFullScreen || el.webkitCancelFullScreen || el.mozCancelFullScreen || el.msCancelFullScreen).call(el)
		},


		play : function(e) {
			if (this.isPlaying) {
				this.isPlaying = false
				this.isDisturbed = false
				this.cancelFullScreen()

				this.disturbTimer.close()
				delete this.disturbTimer
				this.$el.off('mousemove', this.onAnyActionWhenPlaying, this)

				this.playInterval.close()
				delete this.playInterval
			}
			else{
				this.isPlaying = true
				this.fullscreen()

				this.disturbTimer = new ff.Timer(2000, function () {
					this.isDisturbed = false
				}, this)
				this.$el.on('mousemove', this.onAnyActionWhenPlaying, this)

				this.newPlaying()
			}

			Vue.flush()
			this.adjustBody()
			this.adjustScale()
		},


		onAnyActionWhenPlaying : function () {
			this.isDisturbed = true
			this.disturbTimer.reset()
		},


		onEnterBar: function () {
			if (this.disturbTimer) {
				this.disturbTimer.pause()
			}
		},


		onLeaveBar: function () {
			if (this.disturbTimer) {
				this.disturbTimer.resume()
			}
		},


		newPlaying : function(time) {
			if (time > 0) {
				ff.store.set('viewer_play_interval_time', time)
			}
			else {
				time = this.playIntervalTime
			}

			if (this.playInterval) {
				this.playInterval.setMs(time)
			}
			else {
				this.playInterval = new ff.Interval(time, true, this.playNext, this)
			}
		},


		playNext : function () {
			this.view(this.index + 1)
		}	
	},


	watch: {
		data: function () {
			this.thumbDataStart = -1
			this.view(this.index)
		},


		hidden: function (hidden) {
			if (hidden) {
				this.$trigger('hide')
			}
			else {
				this.$trigger('show')
			}
		},
	}
})



//share component
Vue.component('sharer', {

	replace: true,

	template: '<div class="sharer">'+
		'<ul f-show="ff.lang.currentLanguage === \'zh\'">'+
			'<li class="sharer-weixin"></li>'+
			'<li class="sharer-douban" f-tip="getShareTip(\'豆瓣\')"></li>'+
			'<li class="sharer-weibo" f-tip="getShareTip(\'微博\')"></li>'+
		'</ul>'+
		'<ul f-show="ff.lang.currentLanguage !== \'zh\'">'+
			'<li class="sharer-google" f-tip="getShareTip(\'Google\')"></li>'+
			'<li class="sharer-facebook" f-tip="getShareTip(\'Facebook\')"></li>'+
			'<li class="sharer-twitter" f-tip="getShareTip(\'Twitter\')"></li>'+
		'</ul>'+
	'</div>',

	props: ['share-url', 'share-title', 'share-text', 'weixin-img', 'weixin-img-size'],

	data: {
		links: {
			google : {
				url: 'https://plus.google.com/share?url={url}&t={title}',
				width: 500,
				height: 490,
			},

			facebook : {
				url: 'http://www.facebook.com/sharer/sharer.php?u={url}&title={title}',
				width: 673,
				height: 374,
			},

			twitter : {
				url: 'https://twitter.com/intent/tweet?url={url}&text={title}',
				width: 548,
				height: 430,
			},

			weixin: {
				img : 'https://airmore.cn/do.php?action=qrcode&data={url}&size=5',
			},

			douban: {
				url : 'http://www.douban.com/share/service?href={url}&name={title}&text={text}',
			},

			weibo: {
				url : 'http://service.weibo.com/share/share.php?url={url}&title={title}&searchPic=true',
			},
		},

		weixinImgSize: 125,
	},

	methods: {
		getShareTip: function (name) {
			return this.$filter('trans')('share_to_', name)
		},

		getShareLink: function (name, url, title) {
			if (name === 'weixin' && this.weixinImg) {
				return this.weixinImg
			}

			var link = this.links[name].url || this.links[name].img
			var text = this.shareText

			link = link.replace(/\{(\w+)\}/g, function (m0, m1) {
				if (m1 === 'url') {
					return encodeURIComponent(url)
				}
				else if (m1 === 'title') {
					return encodeURIComponent(title)
				}
				else if (m1 === 'text') {
					return encodeURIComponent(text)
				}
			})

			return link
		},

		shareTo: function (name, url, title, btn) {
			url = url || location.href
			title = title || document.title

			var link = this.getShareLink(name, url, title)

			if (this.links[name].url) {
				var width = this.links[name].width
				var height = this.links[name].height
				var params = ''

				if (width && height) {
					params = 'width=' + width + ',height=' + height
				}

				window.open(link, 'share-to-' + name, params)
			}
			else {
				if (!this.weixinTip) {
					var size = this.weixinImgSize
					var imgHTML = '\'<img style="margin: 6px 0; width: ' + size + 'px; height: ' + size + 'px;" src="'+ link +'">\''

					this.weixinTip = this._bindDir('tip', btn, {
						expression: imgHTML,
						raw: imgHTML
					}, Vue.directive('tip'))

					this.weixinTip.showTip()
				}
			}
		}
	},

	ready: function () {
		var re = new RegExp(Object.keys(this.links).join('|'))

		this.$el.on('click', function (e) {
			var name
			var li = e.target.up('li')

			if (li) {
				name = li.className.select(re, 0)
			}

			if (name) {
				this.shareTo(name, this.shareUrl, this.shareTitle, li)
			}
		}, this)

		this.$el.query('li').on('mouseenter', function (e) {
			var li = e.target.up('li')

			if (li) {
				name = li.className.select(re, 0)
			}

			if (name && this.links[name].img) {
				this.shareTo(name, this.shareUrl, this.shareTitle, li)
			}
		}, this)
	}
})


})();